package au.com.explodingsheep.diskDOM;


/**
 * My version of a Attr interface.
 * An Attr must have one child Text node which is where the value
 * of the Attr is stored.
 */


import org.w3c.dom.*;
import au.com.explodingsheep.diskDOM.identifier.*;
//import au.com.explodingsheep.diskDOM.nodeStore.*;
//import au.com.explodingsheep.diskDOM.stringStore.*;
//import au.com.explodingsheep.diskDOM.dtdParser.*;


public class DefaultMyAttr extends DefaultMyNode implements MyAttr
{
  /* Set to true if a value has been explicitly set for this attribute. */
  protected boolean specified = false;
  
  
  /**
   * Constructor.
   */
  
  public DefaultMyAttr( Identifier instanceID,
                  Identifier identifier, 
                  IdentifierIdentifier parent, 
                  IdentifierList children, 
                  String prefix,
                  String namespaceURI,
                  String nodeName,
                  int nodeType,
                  Identifier document,
                  boolean specified,
                  boolean readOnly )
  {
    super( instanceID,
            identifier, 
            parent, 
            children, 
            prefix,
            namespaceURI,
            nodeName,
            nodeType,
            document,
            readOnly );
    this.specified = specified;
  }
  
  
  /**
   * Returns the name of this attribute. 
   */
  
  public String getName()
  {
    String result = super.getNodeName();
    return result;
  }

  
  /**
   * Called by "setNodeValue" and "setDefaultValue" to really 
   * set the value of this attribute.
   * 
   * Note that we do not set the "specified" flag or save the 
   * node to disk. This is up the the caller of this method
   * to do.
   */
  
  protected void realSetNodeValue( String nodeValue )
  {
    /* Get the first child node of this node. It should be a Text node. */
    Text childTextNode = ( Text ) super.getFirstChild();
    if ( childTextNode == null )
    {
      /* No value has been assigned to this Attr yet. Create a */
      /* child Text node to hold the value.                     */
      Document document = super.getOwnerDocument();
      childTextNode = document.createTextNode( nodeValue );
      super.appendChild( childTextNode );
    }
    else
    {
      childTextNode.setNodeValue( nodeValue );
    }
  }
  
  
  /**
   * Replaces any entity references in the passed string with
   * their textual equivilent.
   */
  
  protected String replaceEntitiesWithText( String s )
  {
    if ( s != null )
    {
      /* If the passed string does not contain a '&' character then */
      /* it has no entities, so we don't need to do anything else.  */
      if ( s.indexOf( '&' ) >= 0 )
      {
        Document document = super.getOwnerDocument();
        DocumentType docType = document.getDoctype();
        if ( docType != null )
        {
          int stringLength = s.length();
          int loop = 0;
          while ( loop < stringLength )
          {
            char ch = s.charAt( loop );
            if ( ch != '&' )
            {
              loop++;
            }
            else
            {
              int entityNameStartPos = loop;
              String entityName = "";
              loop++;
              ch = s.charAt( loop );
              while ( ch != ';' )
              {
                entityName += ch;
                loop++;
                ch = s.charAt( loop );
              }
              int entityNameEndPos = loop + 1;
              
              /* Look up the entity. */
              NamedNodeMap entities = docType.getEntities();
              if ( entities != null )
              {
                Entity entity = ( Entity ) entities.getNamedItem( entityName );
                if ( entity != null )
                {
                  /* The entity had better have only one text node. */
                  Node node = entity.getFirstChild();
                  if ( node instanceof CharacterData )
                  {
                    String entityText = node.getNodeValue();
                    String leftString = s.substring(0, entityNameStartPos );
                    String rightString = s.substring( entityNameEndPos );
                    s = leftString + entityText + rightString;
                    stringLength = s.length();
                    loop = entityNameStartPos + entityText.length();
                  }
                  else
                  {
                    /* The node is not a text node. This is a bad thing. */
                  }
                }
                else
                {
                  /* Is the entity one of the standard ones? */
                  /* !!! Do something here.                   */
                }
              }
              else
              {
                /* No entities. This is ok. */
              }
            }
          }
        }
        else
        {
          /* No doctype. This is ok. */
        }
      }
      else
      {
        /* String has no '&'. This is fine. */
      }
    }
    return s;
  }


  /*------------------------- MyAttr methods ------------------------------*/
  
  
  /**
   * Sets the value of ths attribute to the passed value and
   * resets the "specified" flag.
   */
  
  public void setDefaultValue( String defaultValue )
  {
    this.realSetNodeValue( defaultValue );
    this.specified = false;
    super.saveNode();
  }
  
  
  /**
   * Sets the value of ths attribute to the passed value and
   * sets the "specified" flag. This is called when an attribute's
   * value is set for the first time from MyDefaultHandler and
   * we want the value to be set without checking the "readonly"
   * flag.
   */
  
  public void setInitialValue( String defaultValue )
  {
    this.realSetNodeValue( defaultValue );
    this.specified = true;
    super.saveNode();
  }

          
  /*-------------------- Attr methods ---------------------------------*/
  
  
  /**
   * If this attribute was explicitly given a value in the original 
   * document, this is true; otherwise, it is false. Note that the 
   * implementation is in charge of this attribute, not the user. 
   * If the user changes the value of the attribute (even if it ends 
   * up having the same value as the default value) then the 
   * specified flag is automatically flipped to true. To re-specify 
   * the attribute as the default value from the DTD, the user must 
   * delete the attribute. The implementation will then make a new 
   * attribute available with specified set to false and the default 
   * value (if one exists). 
   * In summary: If the attribute has an assigned value in the 
   * document then specified is true, and the value is the assigned 
   * value.If the attribute has no assigned value in the document and 
   * has a default value in the DTD, then specified is false, and the 
   * value is the default value in the DTD.If the attribute has no 
   * assigned value in the document and has a value of #IMPLIED in 
   * the DTD, then the attribute does not appear in the structure 
   * model of the document.If the ownerElement attribute is null 
   * (i.e. because it was just created or was set to null by the 
   * various removal and cloning operations) specified is true. 
   */
  
  public boolean getSpecified()
  {
    return this.specified;
  }

  
  /**
   * On retrieval, the value of the attribute is returned as a string. 
   * Character and general entity references are replaced with their 
   * values. See also the method getAttribute on the Element interface. 
   * On setting, this creates a Text node with the unparsed contents 
   * of the string. I.e. any characters that an XML processor would 
   * recognize as markup are instead treated as literal text. See also 
   * the method setAttribute on the Element interface.   
   */
  
  public String getValue()
  {
    String result = this.getNodeValue();
    result = this.replaceEntitiesWithText( result );
    return result;
  }
  
  
  /**
   * On retrieval, the value of the attribute is returned as a string. 
   * Character and general entity references are replaced with their 
   * values. See also the method getAttribute on the Element interface. 
   * On setting, this creates a Text node with the unparsed contents 
   * of the string. I.e. any characters that an XML processor would 
   * recognize as markup are instead treated as literal text. See also 
   * the method setAttribute on the Element interface. 
   */
  
  public void setValue(String value) throws DOMException
  {
    this.specified = true;
    this.setNodeValue( value );
  }
  
  
  /**
   * The Element node this attribute is attached to or null if this 
   * attribute is not in use. 
   */
  
  public Element getOwnerElement()
  {
    Element result = null;
    Node node = super.getParentNode();
    if ( node instanceof Element )
    {
      result = ( Element ) node;
    }
    return result;
  }
  
  
  /*-------------------- Overridden Node methods ------------------------*/
  
  
  /**
   * Returns the parent node of this node.
   * Because this is an Attribute it should return null here.
   */
  
  public Node getParentNode()
  {  
    return null;
  }
  
  
  /**
   * Attributes should return null.
   */
  
  public Node getPreviousSibling()
  {
    return null;
  }


  /**
   * Attributes should return null.
   */
  
  public Node getNextSibling()
  {
    return null;
  }
  
  
  /**
   * Returns the value of this attribute.
   */
  
  public String getNodeValue() throws DOMException
  {
    String result = "";
    
    /* Get the first child node of this node. It should be a Text node. */
    Text childTextNode = ( Text ) super.getFirstChild();
    if ( childTextNode != null )
    {
      result = childTextNode.getNodeValue();
    }
    
    return result;
  }  
  
  
  /**
   * Sets the value of this node as text.
   * We store the value in the one and only child Text node of this
   * Attr node.
   */
  
  public void setNodeValue(String nodeValue) throws DOMException
  {
    if ( super.isReadOnly() == false )
    {
      this.realSetNodeValue( nodeValue );
      this.specified = true;
      super.saveNode();
    }
    else
    {
      throw new DOMException( DOMException.NO_MODIFICATION_ALLOWED_ERR, "Attribute is read-only." );
    }
  }
  


  /**
   * Clones this element node.
   */
  
  public Node cloneNode(boolean deep)
  {
    Node result = null;
    
    if ( deep == true )
    {
      result = super.cloneNode( true );
    }
    else
    {
      Document document = super.getOwnerDocument();
      String name = super.getNodeName();
      String value = this.getNodeValue();
      result = document.createAttribute( name );
      result.setNodeValue( value );
    }
        
    return result;
  }
}
