package au.com.explodingsheep.diskDOM;


/**
 * My implementation of the MyCDATASection interface.
 */


import org.w3c.dom.*;
import au.com.explodingsheep.diskDOM.identifier.*;
import au.com.explodingsheep.diskDOM.stringStore.*;


public class DefaultMyCDATASection extends DefaultMyText implements MyCDATASection, HasValueIdentifier
{
  /* Points to the value of this Node as stored in the StringStore. */
  protected Identifier value = null;
  

  /**
   * Constructor.
   */
  
  public DefaultMyCDATASection( Identifier instanceID,
                          Identifier identifier, 
                          IdentifierIdentifier parent, 
                          String prefix,
                          String namespaceURI,
                          String nodeName,
                          int nodeType,
                          Identifier value,
                          Identifier document )
  {
    super( instanceID,
            identifier, 
            parent, 
//            attributes,
            prefix,
            namespaceURI,
            nodeName,
            nodeType,
            value,
            document,
            false );
    this.value = value;
  }


  /*------------------------- Overridden Node methods -------------------------*/
  
  
  /**
   * Returns the value of this node as text.
   */
  
  public String getNodeValue() throws DOMException
  {
    String result = null;
    try
    {
      if ( this.value != null )
      {
        result = super.stringStore.getString( this.value );
      }
    }
    catch ( StringStoreException sse )
    {
      throw new DOMException( super.DOM_EXCEPTION_NUM, sse.getMessage() );
    }
    return result;
  }  
  
  
  /**
   * Sets the value of this node as text.
   */
  
  public void setNodeValue( String nodeValue ) throws DOMException
  {
    try
    {
      super.stringStore.setString( this.value, nodeValue );
    }
    catch ( StringStoreException sse )
    {
      throw new DOMException( super.DOM_EXCEPTION_NUM, sse.getMessage() );
    }
  }
  
  
  /**
   * Return "#cdata-section".
   */
  
  public String getNodeName()
  {
    return "#cdata-section";
  }


  /**
   * Clones this element node.
   */
  
  public Node cloneNode(boolean deep)
  {
    Node result = null;
    
    if ( deep == true )
    {
      result = super.cloneNode( true );
    }
    else
    {
      Document document = super.getOwnerDocument();
      String data = super.getNodeValue();
      result = document.createCDATASection( data );
    }
    
    return result;
  }


  /**
   * A processing instruction instruction has no child nodes so
   * return an empty nodelist.
   */
  
  public NodeList getChildNodes()
  {
    NodeList result = new au.com.explodingsheep.diskDOM.helpers.EmptyNodeList();
    return result;
  }


  /*----------------------- HasValueIdentifier methods -----------------------*/
  
  
  /**
   * Returns the Identifier to the value of the node.
   */

  public au.com.explodingsheep.diskDOM.identifier.Identifier getValueIdentifier()
  {
    return this.value;
  }
}
