package au.com.explodingsheep.diskDOM;


import org.w3c.dom.*;
import au.com.explodingsheep.diskDOM.identifier.*;
import au.com.explodingsheep.diskDOM.stringStore.*;


/**
 * My version of the Text class.
 */

public class DefaultMyProcessingInstruction extends DefaultMyNode implements MyProcessingInstruction, HasValueIdentifier
{
  /* Points to the value of this Node as stored in the StringStore. */
  protected Identifier value = null;
    

  /**
   * Constructor.
   */
  
  public DefaultMyProcessingInstruction( Identifier instanceID,
                                    Identifier identifier, 
                                   IdentifierIdentifier parent, 
                                   String prefix,
                                   String namespaceURI,
                                   String nodeName,
                                   int nodeType,
                                   Identifier value,
                                   Identifier document,
                                   boolean readOnly)
  {
    super( instanceID,
            identifier, 
            parent, 
            null, 
            prefix,
            namespaceURI,
            nodeName,
            nodeType,
            document,
            readOnly );
    this.value = value;
  }


  /*------------------------- Overridden Node methods -------------------------*/
  
  
  /**
   * Returns the value of this node as text.
   */
  
  public String getNodeValue() throws DOMException
  {
    String result = null;
    try
    {
      if ( this.value != null )
      {
        result = super.stringStore.getString( this.value );
      }
    }
    catch ( StringStoreException sse )
    {
      throw new DOMException( super.DOM_EXCEPTION_NUM, sse.getMessage() );
    }
    return result;
  }  
  
  
  /**
   * Sets the value of this node as text.
   */
  
  public void setNodeValue( String nodeValue ) throws DOMException
  {
    super.confirmWriteAccess();

    try
    {
      super.stringStore.setString( this.value, nodeValue );
    }
    catch ( StringStoreException sse )
    {
      throw new DOMException( super.DOM_EXCEPTION_NUM, sse.getMessage() );
    }
  }


  /*------------------------- Overridden Node methods -----------------------*/


  /**
   * Clones this element node.
   */
  
  public Node cloneNode(boolean deep)
  {
    Node result = null;
    
    if ( deep == true )
    {
      result = super.cloneNode( true );
    }
    else
    {
      Document document = super.getOwnerDocument();
      String target = this.getNodeName();
      String data = this.getNodeValue();
      result = document.createProcessingInstruction( target, data );
    }
        
    return result;
  }


  /**
   * A processing instruction instruction has no child nodes so
   * return an empty nodelist.
   */
  
  public NodeList getChildNodes()
  {
    NodeList result = new au.com.explodingsheep.diskDOM.helpers.EmptyNodeList();
    return result;
  }

  
  /*------------------- ProcessingInstruction methods -------------------------*/
  
  
  /**
   * The target of this processing instruction. XML defines this as 
   * being the first token following the markup that begins the 
   * processing instruction.
   */ 
  
  public String getTarget()
  {
    String result = super.getNodeName();
    return result;
  }
  

  /**
   * The content of this processing instruction. This is from the 
   * first non white space character after the target to the 
   * character immediately preceding the ?>.   
   */
  
  public String getData()
  {
    String result = this.getNodeValue();
    return result;
  }


  /**
   * The content of this processing instruction. This is from the 
   * first non white space character after the target to the 
   * character immediately preceding the ?>. 
   */
  
  public void setData(String data) throws DOMException
  {
    this.setNodeValue( data );
  }


  /*----------------------- HasValueIdentifier methods -----------------------*/
  
  
  /**
   * Returns the Identifier to the value of the node.
   */

  public au.com.explodingsheep.diskDOM.identifier.Identifier getValueIdentifier()
  {
    return this.value;
  }
}
