package au.com.explodingsheep.diskDOM;


import org.w3c.dom.*;
import au.com.explodingsheep.diskDOM.identifier.*;


/**
 * My version of the Text class.
 */

public class DefaultMyText extends DefaultMyCharacterData implements MyText, HasValueIdentifier
{
  /**
   * Constructor.
   */
  
  public DefaultMyText( Identifier instanceID,
                  Identifier identifier, 
                  IdentifierIdentifier parent, 
                  String prefix,
                  String namespaceURI,
                   String nodeName,
                  int nodeType,
                  Identifier value,
                  Identifier document,
                  boolean readOnly )
  {
    super( instanceID,
            identifier, 
            parent, 
            null,
//            attributes,
            prefix,
            namespaceURI,
            nodeName,
            nodeType,
            value,
            document,
            readOnly );
  }

  
  /**
   * Breaks this node into two nodes at the specified offset, 
   * keeping both in the tree as siblings. After being split, 
   * this node will contain all the content up to the offset 
   * point. A new node of the same type, which contains all the 
   * content at and after the offset point, is returned. If the 
   * original node had a parent node, the new node is inserted 
   * as the next sibling of the original node. When the offset 
   * is equal to the length of this node, the new node has no 
   * data. 
   */
  
  public Text splitText(int offset) throws DOMException
  {
    Text result = null;
    
    /* Get the current text value of the this node. */
    String value = super.getNodeValue();

    /* Make sure the passed split position is not greater than the length */
    /* of the string.                                                      */
    int valueLength = value.length();
    if ( ( offset >= 0 ) && ( offset <= valueLength ) )
    {
      /* Split the text. */
      String leftSplit = value.substring( 0, offset );
      String rightSplit = value.substring( offset );
      
      /* Re-set the value in this node to be just the left split. */
      super.setNodeValue( leftSplit );
      
      /* Create a new node for the right split. */
      Document document = super.getOwnerDocument();
      result = document.createTextNode( rightSplit );
      
      /* Insert the new node after this node. */
      Node node = super.getNextSibling();
      if ( node != null )
      {
        node.insertBefore( result, node );
      }
      else
      {
        Node parent = super.getParentNode();
        parent.appendChild( result );
      }
    }
    else
    {
      throw new DOMException( DOMException.INDEX_SIZE_ERR, "Offset " + offset + " is outside of string \"" + value + "\" - length " + valueLength + "." );
    }
    
    return result;
  }


  /*------------------------- Overridden Node methods -----------------------*/

  
  /**
   * Return "#text".
   */
  
  public String getNodeName()
  {
    return "#text";
  }
  

  /**
   * Return "#text".
   */
  
  public String getLocalName()
  {
    return "#text";
  }


  /**
   * Clones this element node.
   */
  
  public Node cloneNode(boolean deep)
  {
    Node result = null;
    
    if ( deep == true )
    {
      result = super.cloneNode( true );
    }
    else
    {
      Document document = super.getOwnerDocument();
      String value = this.getNodeValue();
      result = document.createTextNode( value );
    }
        
    return result;
  }


  /**
   * A processing instruction instruction has no child nodes so
   * return an empty nodelist.
   */
  
  public NodeList getChildNodes()
  {
    NodeList result = new au.com.explodingsheep.diskDOM.helpers.EmptyNodeList();
    return result;
  }


  /*----------------------- HasValueIdentifier methods -----------------------*/
  
  
  /**
   * Returns the Identifier to the value of the node.
   */

  public au.com.explodingsheep.diskDOM.identifier.Identifier getValueIdentifier()
  {
    return this.value;
  }
}
