package au.com.explodingsheep.diskDOM;


import org.w3c.dom.*;
import au.com.explodingsheep.diskDOM.helpers.ErrorHandler;
import au.com.explodingsheep.diskDOM.identifier.*;
import au.com.explodingsheep.diskDOM.nodeStore.*;


/**
 * This class implements the org.w3c.dom.NodeList interface and
 * acts as a wrapper to the identifer.IdentifierList.
 */

public class IdentifierListNodeList implements NodeList
{
  /* The IdentifierList that points to all the MyNodes we want */
  /* to retrieve.                                               */
  protected IdentifierList identifierList = null;
  
  /* We use this to load nodes from the persistent place and */
  /* return them here.                                       */
  protected NodeStore nodeStore = null;
  
  
  /**
   * Constructor.
   */

  public IdentifierListNodeList( IdentifierList identifierList, NodeStore nodeStore )
  {
    this.identifierList = identifierList;
    this.nodeStore = nodeStore;
  }
  
  
  /**
   * Returns the IdentifierList we are wrapping.
   */
  
  public IdentifierList getIdentifierList()
  {
    return this.identifierList;
  }
  
  
  /*-------------------------------- NodeList methods -----------------------------*/
  
  /**
   * Returns a Node from the list.
   */
  
  public Node item( int index )
  {
    Node result = null;

    try
    {
      Identifier identifier = this.identifierList.get( index );
      result = this.nodeStore.loadNode( identifier );
    }
    catch ( IdentifierListException ile )
    {
      ErrorHandler.throwException( ile );
    }
    catch ( NodeStoreException nse )
    {
      ErrorHandler.throwException( nse );
    }

    return result;
  }
  
  
  /**
   * Returns the number of Nodes in this list.
   */
  
  public int getLength()
  {
    int result = 0;
    try
    {
      result = this.identifierList.size();
    }
    catch ( IdentifierListException ile )
    {
      ErrorHandler.throwException( ile );
    }
    return result;
  }
}
