package au.com.explodingsheep.diskDOM;


import org.w3c.dom.*;
import au.com.explodingsheep.diskDOM.identifier.*;
import au.com.explodingsheep.diskDOM.nodeStore.*;
import au.com.explodingsheep.diskDOM.stringStore.*;


/**
 * My version of the DOMImplementation interface.
 * It is the central point that all de-serializable objects come to
 * when they want to get references to any storage object, like
 * NodeStore or StringStore.
 */

public interface MyDOMImplementation extends DOMImplementation
{
  /**
   * Closes the DOMInstances and any resources it has open.
   */
  
  public void close() throws MyDOMImplementationException;
  
  
  /**
   * Returns an IdentifierIdentifierStore.
   */
  
  public IdentifierStore  getIdentifierStore();
  
  
  /**
   * Returns a NodeStore.
   */
  
  public NodeStore getNodeStore();
  
  
  /**
   * Returns a IdentifierListManager.
   */
  
  public IdentifierListStore getIdentifierListStore();
  
  
  /**
   * Returns a StringStore.
   */
  
  public StringStore getStringStore();
  
  
  /**
   * Returns the Identifier for this Instance.
   */
  
  public Identifier getIdentifier();
  
  
  /**
   * Sets the Identifier to associate with this DOMImplementation.
   * All the MyNodes created will bet set with this identifier.
   */
  
  public void setIdentifier( Identifier identifier );
  
    
  /**
   * Returns a IdentifierGenerator used to create Identifiers for
   * the Nodes in the Document.
   */
  
  public IdentifierGenerator getIdentifierGenerator();

  
  /**
   * If a Document has been created by an instance of this class
   * then it will have been cached by the same instance. If so it
   * can be returned here given its identifier. Returns null if
   * the document is not cached, possibly because a previously 
   * processed DOM file is being opened.
   */
  
  public MyDocument getCachedDocument( Identifier documentIdentifier );

  
//  /**
//   * Adds a Document to the list of cached Documents.
//   */
//  
//  public void addDocument( MyDocument document );


  /**
   * Loads and returns the document with the passed Identifier.
   */
  
  public abstract MyDocument getDocument( String documentName ) throws NodeStoreException, MyDOMImplementationException;
  
  
  /**
   * Returns the names of all the MyDocuments in this
   * DocumentBuilder.
   */
  
  public abstract String[] getDocumentNames() throws MyDOMImplementationException;
}
