package au.com.explodingsheep.diskDOM;


import java.util.*;
import au.com.explodingsheep.diskDOM.identifier.*;


/**
 * This class is where MyDOMImplementation instances are registered. 
 * It is a common point that objects can come to if they have a 
 * MyDOMImplementation Identifier and need to get a reference to the  
 * MyDOMImplementation itself.
 */

public class MyDOMImplementationManager
{
  protected static List instances = new ArrayList();
  
  
  /**
   * Registers a new instance with us and returns a unique identifier for it.
   */
  
  public static Identifier addInstance( MyDOMImplementation domImplementation )
  {
    Identifier result = null;
    if ( instances.contains( domImplementation ) == false )
    {
      int numInstances = instances.size();
      instances.add( domImplementation );
      result = new IntIdentifier( numInstances );
    }
    else
    {
      result = domImplementation.getIdentifier();
    }
    return result;
  }
  
  
  /**
   * Returns an instance given the passed ID.
   */
  
  public static MyDOMImplementation getInstance( Identifier anIdentifier )
  {
    MyDOMImplementation result = null;
    int loop = 0;
    int numInstances = instances.size();
    while ( loop < numInstances )
    {
      MyDOMImplementation domImplementation = ( MyDOMImplementation ) instances.get( loop );
      Identifier identifier = domImplementation.getIdentifier();
      if ( identifier.equals( anIdentifier ) == true )
      {
        result = domImplementation;
        loop = numInstances;
      }
      else
      {
        loop++;
      }
    }
    return result;
  }
  
  
  /**
   * Removes an instance of a DOMIdentifier from this shared list.
   */
  
  public static void removeInstance( MyDOMImplementation domImplementation )
  {
    instances.remove( domImplementation );
  }
  
  
  /**
   * Closes all instances of the MyDOMImplementations held in
   * this manager.
   */
  
  public static void close() throws MyDOMImplementationException
  {
    int numInstances = instances.size();
    for ( int loop = 0; loop < numInstances; loop++ )
    {
      MyDOMImplementation di = ( MyDOMImplementation ) instances.get( loop );
      di.close();
      MyDOMImplementationManager.removeInstance( di );
    }
  }
}
