package au.com.explodingsheep.diskDOM;


import javax.xml.parsers.*;
import org.w3c.dom.*;
import junit.framework.*;


/**
 * Test MyDocument to make sure it creates all the various Node types.
 */

public class TestDocument extends TestCase
{
  /* Identifies the above DOMInstance. */
  protected static int instanceID = 0;
  
  protected static DocumentBuilder db = null;
  

  /**
   * Constructor for TestDocument.
   * @param arg0
   */
  public TestDocument(String arg0)
  {
    super(arg0);
  }
  

  public void testCreateDocumentBuilder() throws Exception
  {
    DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
    TestDocument.db = dbf.newDocumentBuilder();

    super.assertTrue( true );
  }


  public void testCreateElementAndAttributes() throws Exception
  {
    boolean result = false;

    Document document = TestDocument.db.newDocument();    
    Element element = document.createElement( "TestElement" );
    Text text = document.createTextNode( "asdfgh" );
    String s = text.getNodeValue();
    if ( s.equals( "asdfgh" ) )
    {
      Attr attr = document.createAttribute( "TestAttribute" );
      if ( attr.getSpecified() == false )
      {
        if ( attr.getNodeType() == Node.ATTRIBUTE_NODE )
        {
          attr.setValue( "def" );
          s = attr.getValue();
          if ( s.equals( "def" ) )
          {
            if ( attr.getSpecified() == true )
            {
              /* Can we join the attribute to the element? */
              element.setAttribute( "ghi", "jkl" );
              NamedNodeMap nnp = element.getAttributes();
              if ( nnp.getLength() == 1 )
              {
                Attr a2 = element.getAttributeNode( "ghi" );
                if ( a2 != null )
                {
                  s = a2.getNodeValue();
                  if ( ( s != null ) && ( s.equals( "jkl" ) == true ) )
                  {
                    Attr newAttr = document.createAttribute( "NewAttr" );
                    s = newAttr.getNodeValue();
                    if ( s.equals( "" ) )
                    {
                      result = true;
                    }
                  }
                }
              }
            }
          }
        }
      }
    }
    
    super.assertTrue( result );
  }
  

  public void testCreateElementAndChildNodes() throws Exception
  {
    boolean result = false;
    
    Document document = TestDocument.db.newDocument();
    Element element = document.createElement( "TestElement2" );
    Element child1 = document.createElement( "ChildElement1" );
    Element child2 = document.createElement( "ChildElement2" );
    Element child3 = document.createElement( "ChildElement3" );

    element.appendChild( child2 );
    element.insertBefore( child1, child2 );
    element.appendChild( child3 );
    
    NodeList children = element.getChildNodes();
    if ( children.getLength() == 3 )
    {
      element.removeChild( child2 );
      if ( children.getLength() == 2 )
      {
        element.insertBefore( child2, child3 );
        if ( children.getLength() == 3 )
        {
          Node childNode = children.item( 0 );
          if ( childNode.getNodeName().equals( "ChildElement1" ) )
          {
            childNode = children.item( 1 );
            if ( childNode.getNodeName().equals( "ChildElement2" ) )
            {
              childNode = children.item( 2 );
              if ( childNode.getNodeName().equals( "ChildElement3" ) )
              {
                result = true;
              }
            }
          }
        }
      }
    }
    
    super.assertTrue( result );
  }


  /**
   * Tests moving up and down a few levels of nodes.
   */
  
  public void testTraversingTree() throws Exception
  {
    boolean result = false;
    
    Document document = TestDocument.db.newDocument();
    Element parent = document.createElement( "TestElement3" );
    Element child = document.createElement( "Child" );
    Element grandChild = document.createElement( "GrandChild" );
    parent.appendChild( child );
    child.appendChild( grandChild );
    document.appendChild( parent );
    
    Node e = grandChild.getParentNode();
    if ( e.equals( child ) )
    {
      e = child.getParentNode();
      if ( e.equals( parent ) )
      {
        e = parent.getParentNode();
        if ( e.equals( document ) )
        {
          result = true;
        }
      }
    }
    
    super.assertTrue( result );
  }
  
  
  
  /**
   * Tests the Element method "getElementsByTagName".
   */
  
  public void testGetElementsByTagName() throws Exception
  {
    boolean result = false;

    Document document = TestDocument.db.newDocument();
    
    /* Create one child node off of the document then 10 grandchildren,  */
    /* each with children of their own.                                   */
    Element rootElement = document.createElement( "Root" );
    document.appendChild( rootElement );
    for ( int loop = 0; loop < 10; loop++ )
    {
      Element newElement1 = document.createElement( "Child1" );
      Element newElement2 = document.createElement( "Child2" );
      Element newElement3 = document.createElement( "Child3" );
      newElement1.appendChild( newElement2 );
      newElement2.appendChild( newElement3 );
      rootElement.appendChild( newElement1 );
    }

    /* Now "getElementsByTagName"  should say there are 10 */
    /* instances of "Child2" elements.                     */
    Element e = ( Element ) document.getFirstChild();
    NodeList nl = e.getElementsByTagName( "Child2" );
    if ( nl != null )
    {
      if ( nl.getLength() == 10 )
      {
        /* Do the same test with Document (it also has the same */
        /* method).                                              */
        NodeList nl2 = document.getElementsByTagName( "Child3" );
        if ( nl2 != null )
        {
          if ( nl2.getLength() == 10 )
          {
            result = true;
          }
        }
      }
    }
    
    super.assertTrue( result );
  }
  
  
  /**
   * When a child is re-added to a parent it should be moved to the
   * end of the child list.
   */
  
  public void testRemovingAndAddingChildren() throws Exception
  {
    boolean result = false;

    Document document = TestDocument.db.newDocument();
    
    /* Create one child node off of the document then 10 grandchildren,  */
    /* each with children of their own.                                   */
    Element rootElement = document.createElement( "Root" );
    document.appendChild( rootElement );
    Element newElement1 = document.createElement( "Child1" );
    Element newElement2 = document.createElement( "Child2" );
    Element newElement3 = document.createElement( "Child3" );
    rootElement.appendChild( newElement1 );
    rootElement.appendChild( newElement2 );
    rootElement.appendChild( newElement3 );
    
    rootElement.appendChild( newElement1 );

    Node node = rootElement.getFirstChild();
    if ( node.equals( newElement2 ) )
    {
      node = rootElement.getLastChild();
      if ( node.equals( newElement1 ) )
      {
        result = true;
      }
    }
    
    super.assertTrue( result );
  }
  
  
  /**
   * Tests doing a shallow clone of an element.
   */
  
  public void testShallowCloneElement() throws Exception
  {
    boolean result = false;
    
    Document document = TestDocument.db.newDocument();
    Element e1 = document.createElement( "Element1" );
    e1.setNodeValue( "abc" );
    e1.appendChild( document.createElement( "Element2" ) );
    
    Element clonedElement = ( Element ) e1.cloneNode( false );
    String clonedElementName = clonedElement.getNodeName();
    if ( clonedElementName.equals( "Element1" ) )
    {
      String clonedElementValue = clonedElement.getNodeValue();
      if ( clonedElementValue == null )
      {
        NodeList children = clonedElement.getChildNodes();
        if ( children.getLength() == 0 )
        {
          String clonedNodeValue = clonedElement.getNodeValue();
          if ( clonedNodeValue == null )
          {
            result = true;
          }
        }
      }
    }
    
    super.assertTrue( result );
  }
  

//  /**
//   * Tests doing a deep clone of an element.
//   */
//  
//  public void testDeepCloneElement() throws Exception
//  {
//    boolean result = false;
//    
//    Document document = this.db.newDocument();
//    Element e1 = document.createElement( "Element1" );
//    Text textNode = document.createTextNode( "abc" );
//    e1.appendChild( textNode );
//    e1.setAttribute( "TestAttribute1", "def" );
//    Element e2 = document.createElement( "Element2" );
//    textNode = document.createTextNode( "ghi" );
//    e2.appendChild( textNode );
//    e2.setAttribute( "TestAttribute2", "jkl" );
//    e1.appendChild( e2 );
//    
//    Element clonedElement = ( Element ) e1.cloneNode( true );
//    if ( clonedElement.getNodeName().equals( "Element1" ) )
//    {
//      String clonedElementValue = clonedElement.getNodeValue();
//      if ( clonedElementValue.equals( "abc" ) )
//      {
//        Attr attr = clonedElement.getAttributeNode( "TestAttribute1" );
//        if ( attr != null )
//        {
//          if ( attr.getNodeValue().equals( "def" ) )
//          if ( clonedElement.getAttributes().getLength() == 1 )
//          {
//            NodeList children = clonedElement.getChildNodes();
//            if ( children.getLength() == 2 ) // The child Element node and the child Text node.
//            {
//              Element clonedChildElement = ( Element ) children.item( 1 );
//              if ( clonedChildElement.getNodeName().equals( "Element2" ) )
//              {
//                if ( clonedChildElement.getNodeValue().equals( "ghi" ) )
//                {
//                  Attr attr2 = clonedChildElement.getAttributeNode( "TestAttribute2" );
//                  if ( attr2.getNodeValue().equals( "jkl" ) )
//                  {
//                    result = true;
//                  }
//                }
//              }
//            }
//          }
//        }
//      }
//    }
//    
//    super.assertTrue( result );
//  }
          
  
  static Test suite()
  {
    TestSuite result = new TestSuite();
    
    result.addTest( new TestDocument( "testCreateDocumentBuilder" ) );
    result.addTest( new TestDocument( "testCreateElementAndAttributes" ) );
    result.addTest( new TestDocument( "testCreateElementAndChildNodes" ) );
    result.addTest( new TestDocument( "testTraversingTree" ) );
    result.addTest( new TestDocument( "testGetElementsByTagName" ) );
    result.addTest( new TestDocument( "testRemovingAndAddingChildren" ) );
    result.addTest( new TestDocument( "testShallowCloneElement" ) );
//    result.addTest( new TestDocument( "testDeepCloneElement" ) );
    
    return result;
  }


  public static void main(String[] args)
  {
    junit.textui.TestRunner.run(TestDocument.class);
  }
}
