package au.com.explodingsheep.diskDOM;


import java.util.*;
import java.io.*;
import javax.xml.parsers.*;
import javax.xml.transform.*;
import javax.xml.transform.dom.*;
import javax.xml.transform.stream.*;
import org.w3c.dom.*;
import au.com.explodingsheep.diskDOM.documentBuilder.*;


/**
 * We want to be able to load a DOM tree from a SAX source.
 * We are going to use a Transformer to do this.
 */

public class TestTransformer 
{
  /**
   * Serializes one object to the passed filename.
   */
  
  protected static void serializeObject( Object o, String fileName ) throws Exception
  {
    FileOutputStream fos = new FileOutputStream( fileName );
    try
    {
      ObjectOutputStream oos = new ObjectOutputStream( fos );
      oos.writeObject( o );
    }
    finally
    {
      fos.close();
    }
  }

  
  /**
   * Transforms XML data using the standard Transformer.
   */
  
  public static void main( String args[] )  
  {
    try
    {
      Date startDate = new Date();
      System.out.println( "Start time = " + startDate.toString() );
      
      /* Get a DocumentBuilder that we will use to get the Document instance. */
//      DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
//      dbf.setIgnoringElementContentWhitespace( true );
//      DocumentBuilder db = dbf.newDocumentBuilder();
//      System.out.println( "DocumentBuilder = " + db.getClass() );

      MyDocumentBuilderFactory dbf = ( MyDocumentBuilderFactory ) DocumentBuilderFactory.newInstance();
      dbf.setStorageFileName( "/tmp/MyDOM/TestTransformer.pdom" );
      dbf.setIgnoringElementContentWhitespace( true );
      MyDocumentBuilder db = ( MyDocumentBuilder ) dbf.newDocumentBuilder();
      System.out.println( "DocumentBuilder = " + db.getClass() );

      /* Create a Document and import an XML into it. */
//      InputStream is = new FileInputStream( "/tmp/TestXMLFiles/TmpFromHostSmall.xml" );
      InputStream is = new FileInputStream( "/tmp/TestXMLFiles/TmpFromHostMedium.xml" );
      try
      {
        Document srcDocument = db.parse( is );
        System.out.println( "Document = " + srcDocument.getClass() );
    
        javax.xml.transform.TransformerFactory tFactory = javax.xml.transform.TransformerFactory.newInstance();
        System.out.println( "TransformerFactory = " + tFactory.getClass() );


         Date loadDate = new Date();
        System.out.println( "Load time  = " + loadDate.toString() );
        

        /* Now we want to transform the source DOM into a file */
        /* using a real XSL script.                             */
        Source xslSource = new StreamSource( new FileInputStream( "/tmp/TestXMLFiles/FROMHOST-DVI.xsl" ) );
        Transformer transformer2 = tFactory.newTransformer( xslSource );
        System.out.println( "Transformer = " + transformer2.getClass() );
        
        DOMSource domSource = new DOMSource( srcDocument );

        StreamResult streamResult = new StreamResult( new FileOutputStream( "/tmp/MyDOM/testdvi.xml" ) );

//        Document destDocument = db.newDocument();
//        DOMResult domResult = new DOMResult( destDocument );

        transformer2.transform( domSource, streamResult );

         Date endDate = new Date();
        System.out.println( "End time   = " + endDate.toString() );
        
        
//        Node node = srcDocument.getFirstChild();
//        printNode( 0, node );
      }
      finally
      {
        is.close();
        db.close();
      }
    }    
    catch ( Exception e )
    {
      e.printStackTrace();
    }
  }
}
