package au.com.explodingsheep.diskDOM;


import org.w3c.dom.*;
import java.util.*;


/**
 * This is a quick and dirty implementation of NamedNodeMap that
 * does not stay up to date with changes to the DOM tree.
 * It is used by MyDocumentType.
 */

public class UnchangingNamedNodeMap implements NamedNodeMap
{
  /* Holds the list of nodes. */
  protected List nodes = new ArrayList();
  
  
  /**
   * Constructor.
   */
  
  public UnchangingNamedNodeMap()
  {
  }
  
  
  /**
   * Adds a node to the list.
   */
  
  public void add( Node node )
  {
    this.nodes.add( node );
  }


  /**
   * Returns the index of the node whose name is passed.
   */
  
  public int getNamedItemIndex(String namespaceURI, String localName)
  {
    int result = -1;
    int numNodes = this.getLength();
    int loop = 0;
    while ( loop < numNodes )
    {
      Node node = ( Node ) this.nodes.get( loop );
      boolean b1 = false;
      boolean b2 = false;
      if ( namespaceURI != null )
      {
        String s1 = node.getPrefix();
        if ( s1.equals( namespaceURI ) )
        {
          b1 = true;
        }
      }
      else
      {
        b1 = true;
      }
      
      String s2 = node.getLocalName();
      if ( s2.equals( localName ) )
      {
        b2 = true;
      }
      
      if ( ( b1 == true ) && ( b2 == true ) )
      {
        result = loop;
        loop = numNodes;
      }
      else
      {
        loop++;
      }
    }
    return result;
  }
  
  
  /*----------------------- NamedNodeMap methods ----------------------------*/
    
    
  /**
   * The number of nodes in this map. 
   */

  public int getLength() 
  {
    int result = this.nodes.size();
    return result;
  }
  
  
  /**
   * Retrieves a node specified by name. 
   */
          
  public Node getNamedItem(String name) 
  {
    Node result = null;
    int index = this.getNamedItemIndex( null, name );
    if ( index >= 0 )
      result = ( Node ) this.nodes.get( index );
    return result;
  }
  

  /**
   * Retrieves a node specified by local name and namespace URI.
   */
  
  public Node getNamedItemNS(String namespaceURI, String localName)
  {
    Node result = null;
    int index = this.getNamedItemIndex( namespaceURI, localName );
    if ( index >= 0 )
      result = ( Node ) this.nodes.get( index );
    return result;
  }

  
  /**
   * Returns the indexth item in the map. 
   */
          
  public Node item(int index) 
  {
    Node result = ( Node ) this.nodes.get( index );
    return result;
  }
  
          
  /**
   * Removes a node specified by name. 
   */
  
  public Node removeNamedItem(String name)
  {
    Node result = null;
    int index = this.getNamedItemIndex( null, name );
    if ( index >= 0 )
    {
      result = ( Node ) this.nodes.get( index );
      this.nodes.remove( index );
    }
    return result;
  }
  
          
  /**
   * Removes a node specified by local name and namespace URI. 
   */
      
  public Node removeNamedItemNS(String namespaceURI, String localName) 
  {
    Node result = null;
    int index = this.getNamedItemIndex( namespaceURI, localName );
    if ( index >= 0 )
    {
      result = ( Node ) this.nodes.get( index );
      this.nodes.remove( index );
    }
    return result;
  }

            
  /**
   * Adds a node using its nodeName attribute. 
   * !!! Currently we do nothing here.
   */
  
  public Node setNamedItem(Node arg)
  {
    return null;
  }
  
            
  /**
   * Adds a node using its nodeName attribute. 
   * !!! Currently we do nothing here.
   */

  public Node setNamedItemNS(Node arg) 
  {
    return null;
  }
}
