package au.com.explodingsheep.diskDOM.documentBuilder;


import javax.xml.parsers.*;
import au.com.explodingsheep.diskDOM.*;
import au.com.explodingsheep.diskDOM.simpleFileSystem.*;
import au.com.explodingsheep.diskDOM.identifier.*;


/**
 * My version of a DocumentBuilderFactory. This it what external code 
 * will most likely use to create Document instances.
 * The SimpleFileSystem stores all its data to one file, whose name 
 * is passed in the "setStorageFileName" method. If no filename is 
 * set then everything is written to RAM and not stored to disk.
 * 
 * Note that the name of "MyDocumentBuilderFactory" is not really correct,
 * it should be "DefaultMyDocumentBuilderFactory" isntead.
 */

public class MyDocumentBuilderFactory extends DocumentBuilderFactory
{
  /* The name of the file to write all XML DOM data to. If null then */
  /* write to RAM.                                                   */
  protected String storageFileName = null;
  
  protected boolean ignoringElementContentWhitespace = false;
  

  /**
   * Constructor.
   */
  
  public MyDocumentBuilderFactory()
  {
    super();
  }
  

  /**
   * Sets whether the DocumentBuilder we create in "newDocumentBuilder"
   * writes its data to disk.
   * If we pass null here the DocumentBuilder will write its nodes to RAM.
   */
  
  public void setStorageFileName( String storageFileName )
  {
    this.storageFileName = storageFileName;
  }
  
  
  /**
   * Gets whether the DocumentBuilder we create in "newDocumentBuilder"
   * writes its data to disk, or to RAM.
   */
  
  public String getStorageFileName()
  {
    return this.storageFileName;
  }
  

  /*-----------------------------  DocumentBuilderFactory methods ------------------*/
  
  
  /**
   * We pass this through to the parent.
   */
  
  public Object getAttribute(String name) throws IllegalArgumentException
  {
    Object result = null;
    return result;
  }
  

  /**
   * Does nothing.
   */
  
  public void setAttribute(String name, Object value) throws IllegalArgumentException
  {
  }
  
  
  /**
   * Creates a new instance of a DocumentBuilder using the 
   * currently configured parameters.
   * 
   * !!! We need to have this method check for an environment variable, or
   * a attribute or feature set in this factory that specifies where to
   * write the PDOM files to.
   * 
   * !!! How do we close the DOMImplementations we create. We need to explicitly
   * call "close" on them.
   */
  
  public DocumentBuilder newDocumentBuilder() throws ParserConfigurationException
  {
    DefaultMyDocumentBuilder result = null;
    BlockIO blockIO = null;

    try
    {
      if ( ( this.storageFileName != null ) && ( this.storageFileName.equals( "" ) == false )  )
      {
        blockIO = new FileBlockIO( this.storageFileName );
      }
      else
      {
        blockIO = new RAMBlockIO();
      }        

      /* Create a DOMImplementation and register it with the      */
      /* DOMImplementationManager. Whenever a node is             */
      /* de-serialized it goes to the DOMImplementationManager to */
      /* to get the DOMImplementation it needs that in turn       */
      /* returns the resource managers the node needs to operate. */
      /*                                                          */
      /* !!! Note that we aren't setting the identifier returned  */
      /* from MyDOMImplementationManager to the                   */
      /* MyDOMImplementation. This is because it has already been */
      /* set in the MyDOMImplementationManager itself.             */
      MyDOMImplementation myDOMImplementation = new DefaultMyDOMImplementation( blockIO );
      Identifier domImplementationIdentifier = MyDOMImplementationManager.addInstance( myDOMImplementation );
      result = new DefaultMyDocumentBuilder( myDOMImplementation, ignoringElementContentWhitespace );
    }
    catch ( MyDOMImplementationException mie )
    {
      throw new ParserConfigurationException( mie.getMessage() );
    }
    catch ( BlockIOException bioe )
    {
      throw new ParserConfigurationException( bioe.getMessage() );
    }
    return result;
  }
  
  
  /**
   * Indicates whether or not the factory is configured to produce 
   * parsers which ignore ignorable whitespace in element content. 
   */
  
  public boolean isIgnoringElementContentWhitespace()
  {
    return this.ignoringElementContentWhitespace;
  }

  
  /**
   * Specifies that the parsers created by this factory must 
   * eliminate whitespace in element content (sometimes known 
   * loosely as 'ignorable whitespace') when parsing XML 
   * documents (see XML Rec 2.10). Note that only whitespace 
   * which is directly contained within element content that 
   * has an element only content model (see XML Rec 3.2.1) 
   * will be eliminated. Due to reliance on the content model 
   * this setting requires the parser to be in validating mode. 
   * By default the value of this is set to false.   
   */
  
  public void setIgnoringElementContentWhitespace(boolean ignoringElementContentWhitespace)
  {
    this.ignoringElementContentWhitespace = ignoringElementContentWhitespace;
  }
}
