package au.com.explodingsheep.diskDOM.documentBuilder;


import java.io.*;
import javax.xml.parsers.*;
import org.w3c.dom.*;
import junit.framework.*;
import au.com.explodingsheep.diskDOM.*;


/**
 * Tests the MyXMLReader class.
 */

public class TestMyXMLReader extends TestCase
{

  /**
   * Constructor for TestMyXMLReader.
   * @param arg0
   */
  public TestMyXMLReader(String arg0)
  {
    super(arg0);
  }


  /**
   * Parses a simple XML file that doesn't have a DTD file.
   */
  
  public void testParsingSimpleXMLFile() throws Exception
  {
    boolean result = false;
    
    String xmlString = "<a><b c=\"def\">ghi</b></a>";
    ByteArrayInputStream bais = new ByteArrayInputStream( xmlString.getBytes() );
    DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
    DocumentBuilder db = dbf.newDocumentBuilder();
    Document doc = db.parse( bais );

    Node node = doc.getFirstChild();
    String s = node.getNodeName();
    if ( s.equals( "a" ) )
    {
      node = node.getFirstChild();
      s = node.getNodeName();
      if ( s.equals( "b" ) )
      {
        Element element = ( Element ) node;
        Attr attr = element.getAttributeNode( "c" );
        s = attr.getNodeValue();
        if ( s.equals( "def" ) )
        {
          result = true;
        }
      }
    }
    
    super.assertTrue( result );
  }


  /**
   * Try parsing a more complex XML file containing a DTD
   * and entities.
   */
  
  public void testParsingComplexXMLFile() throws Exception
  {
    boolean result = false;

    /* First we want to write a test simple DTD file to disk so the */
    /* DTDParser can parse it.                                      */
    OutputStream os = new FileOutputStream( "/tmp/test.dtd" );
    try
    {
      os.write( "<!ELEMENT Addressbook (AddressElement)+>\n".getBytes() );
      os.write( "<!ELEMENT AddressElement ( FirstName, LastName, Email, Address?, City?,Zip?, Tel?, Fax? )>\n".getBytes() );
      os.write( "<!ELEMENT Firstname (#PCDATA)>\n".getBytes() );
      os.write( "<!ELEMENT Lastname (#PCDATA)>\n".getBytes() );
      os.write( "<!ELEMENT Email (#PCDATA)>\n".getBytes() );
      os.write( "<!ELEMENT Address (#PCDATA)>\n".getBytes() );
      os.write( "<!ELEMENT City (#PCDATA)>\n".getBytes() );
      os.write( "<!ELEMENT Zip (#PCDATA)>\n".getBytes() );
      os.write( "<!ELEMENT Tel (#PCDATA)>\n".getBytes() );
      os.write( "<!ELEMENT Fax (#PCDATA)>\n".getBytes() );
      os.write( "<!ATTLIST Addressbook Name  CDATA  #REQUIRED  LastUpdate  CDATA  #IMPLIED>\n".getBytes() );
      os.write( "<!ATTLIST AddressElement  AddressID  CDATA  #REQUIRED>\n".getBytes() );
      os.write( "<!ENTITY TestEntity1 \"def\">\n".getBytes() );
      os.write( "<!ENTITY TestEntity2 \"<d><e>fgh</e></d>\">\n".getBytes() );
    }
    finally
    {
      os.close();
    }


    String srcXML = "<!DOCTYPE test SYSTEM \"/tmp/test.dtd\"> <test><b c=\"def\">&TestEntity2;</b></test>";
    ByteArrayInputStream bais = new ByteArrayInputStream( srcXML.getBytes() );
    DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
    DocumentBuilder db = dbf.newDocumentBuilder();
    Document doc = db.parse( bais );

    Node node = doc.getDocumentElement();
    String s = node.getNodeName();
    if ( s.equals( "test" ) )
    {
      node = node.getFirstChild();
      s = node.getNodeName();
      if ( s.equals( "b" ) )
      {
        Element element = ( Element ) node;
        Attr attr = element.getAttributeNode( "c" );
        s = attr.getNodeValue();
        if ( s.equals( "def" ) )
        {
          MyNode myNode = ( MyNode ) node.getFirstChild();
          
          /* This child of node <b> is an entity reference. under it is the <d> node. */
          myNode = ( MyNode ) myNode.getFirstChild();
          
          s = myNode.getNodeName();
          if ( ( s.equals( "d" ) ) && ( myNode.isReadOnly() == true ) )
          {
            myNode = ( MyNode ) myNode.getFirstChild();
            s = myNode.getNodeName();
            if ( ( s.equals( "e" ) ) && ( myNode.isReadOnly() == true ) )
            {
              Text textNode = ( Text ) myNode.getFirstChild();
              s = textNode.getNodeValue();
              if ( s.equals( "fgh" ) )
              {
                result = true;
              }
            }
          }
        }
      }
    }
    
    super.assertTrue( result );
  }
    

  public static Test suite()
  {
    TestSuite result = new TestSuite();
    
    result.addTest( new TestMyXMLReader( "testParsingSimpleXMLFile" ) );
    result.addTest( new TestMyXMLReader( "testParsingComplexXMLFile" ) );
    
    return result;
  }
  

  public static void main(String[] args)
  {
    junit.textui.TestRunner.run(TestMyXMLReader.class);
  }

}
