package au.com.explodingsheep.diskDOM.dtdParser;


/**
 * Implements DTDElement interface.
 */

public class DefaultDTDElement implements DTDElement, java.io.Serializable
{
  protected String name = null;
  protected ChildDTDElementList childList = null;
  protected int dataType = DTDDataTypes.NO_DATA_TYPE;
  protected DTDAttributeList attributes = null;


  /**
   * Constructor used when an element has children.
   */

  public DefaultDTDElement( String name, ChildDTDElementList childList )
  {
    this.name = name;
    this.childList = childList;
  }


  /**
   * Constructor used when an element has no children.
   */

  public DefaultDTDElement( String name, int dataType )
  {
    this.name = name;
    this.dataType = dataType;
  }

  /**
   * Constructor used when an element has children or a datatype.
   */

  public DefaultDTDElement( String name, int dataType, ChildDTDElementList childList )
  {
    this.name = name;
    this.dataType = dataType;
    this.childList = childList;
  }


  /**
   * Sets the list of attributes associated with this element.
   */
  
  public void setAttributes( DTDAttributeList attributes )
  {
    this.attributes = attributes;
  }
      

  /*----------------------- DTDElement methods ---------------------------*/
  

  /**
   * Returns the name of the element.
   */
  
  public String getName()
  {
    return this.name;
  }
  
  
  /**
   * Returns whether the element has data, or children elements.
   */
  
  public boolean hasChildren()
  {
    boolean result = ( this.childList != null );
    return result;
  }
  
  
  /**
   * Returns the datatype of the element (if it has data).
   */
  
  public int getDataType()
  {
    return this.dataType;
  }  
  
  
  /**
   * Returns the list of children elements for this element
   * (if this element has children).
   */
  
  public ChildDTDElementList getChildList()
  {
    return this.childList;
  }
  
  
  /**
   * Returns the attributes for this node, or null if there
   * are none.
   */
  
  public DTDAttributeList getDTDAttributes()
  {
    return this.attributes;
  }
}
