package au.com.explodingsheep.diskDOM.dtdParser;


import org.w3c.dom.*;
import javax.xml.parsers.*;
import java.io.*;
import junit.framework.*;
import au.com.explodingsheep.diskDOM.*;


/**
 * Tests the DTDParser interface.
 */

public class TestDTDParser extends TestCase
{

  /**
   * Constructor for TestDTDParser.
   * @param arg0
   */
  public TestDTDParser(String arg0)
  {
    super(arg0);
  }


  /**
   * Tests the testDTDDefaultDTDParser class.
   */
  
  public void testDTDDefaultDTDParser() throws Exception
  {
    boolean result = false;

    String dtd = "";
    dtd += "<!DOCTYPE testdtd SYSTEM [";
    dtd += "<!ENTITY TestEntity1 \"abcde\">";
    dtd += "<!ENTITY TestEntity2 \"<abc d=\'efg\'>ghi</abc>\">";
    dtd += "<!ELEMENT Addressbook (AddressElement)+>";
    dtd += "<!ELEMENT AddressElement ( FirstName, LastName, Email, Address?, City?,Zip?, Tel?, Fax? )>";
    dtd += "<!ELEMENT Firstname (#PCDATA)>";
    dtd += "<!ELEMENT Lastname (#PCDATA)>";
    dtd += "<!ELEMENT Email (#PCDATA)>";
    dtd += "<!ELEMENT Address (#PCDATA)>";
    dtd += "<!ELEMENT City (#PCDATA)>";
    dtd += "<!ELEMENT Zip (#PCDATA)>";
    dtd += "<!ELEMENT Tel (#PCDATA | City)>";
    dtd += "<!ELEMENT Fax (#PCDATA | Zip)*>";
    dtd += "<!ATTLIST Addressbook Name  CDATA  #REQUIRED  LastUpdate  CDATA  #IMPLIED>";
    dtd += "<!ATTLIST AddressElement  AddressID  CDATA  \"Default attribute\">";
    dtd += "]>";


    DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
    DocumentBuilder db = dbf.newDocumentBuilder();
    MyDocument doc = ( MyDocument ) db.newDocument();
    DOMImplementation di = db.getDOMImplementation();
    
    DTDParser parser = new DefaultDTDParser( doc, di );

    InputStream is = new ByteArrayInputStream( dtd.getBytes() );    
    
    parser.parse( is );
    
    MyDocumentType docType = ( MyDocumentType ) doc.getDoctype();
    
    DTDElement element = docType.getDTDElement( "Addressbook" );
    if ( element != null )
    {
      ChildDTDElementList children = element.getChildList();
      if ( ( children != null ) && ( children.size() == 1 ) )
      {
        ChildDTDElement childElement = children.getChildDTDElement( 0 );
        if ( ( childElement.getMinimumCount() == 1 )
          && ( childElement.getMaximumCount() == 1 ) )
        {
          DTDAttributeList attributes = element.getDTDAttributes();
          if ( ( attributes != null ) && ( attributes.size() == 2 ) )
          {
            DTDAttribute attribute = attributes.getDTDAttribute( 0 );
            if ( ( attribute != null ) && ( attribute.isImplied() == false ) && ( attribute.isRequired() == true ) )
            {

              element = docType.getDTDElement( "AddressElement" );
              if ( element != null )
              {
                attributes = element.getDTDAttributes();
                if ( ( attributes != null ) && ( attributes.size() == 1 ) )
                {
                  attribute = attributes.getDTDAttribute( 0 );
                  if ( attribute != null ) 
                  {
                    String defaultValue = attribute.getDefaultValue();
                    if ( defaultValue.equals( "Default attribute" ) == true )
                    {
                      NamedNodeMap entities = docType.getEntities();
                      Entity entity = ( Entity ) entities.item( 0 );
                      if ( entity.getNodeName().equals( "TestEntity1" ) )
                      {
                        Text child = ( Text ) entity.getFirstChild();
                        if ( child.getNodeValue().equals( "abcde" ) )
                        {
                          result = true;
                        }
                      }
                    }
                  }
                }
              }
            }
          }
        }
      }
    }
    
    super.assertTrue( result );
  }


  public static Test Suite()
  {
    TestSuite result = new TestSuite();
    result.addTest( new TestDTDParser( "testDTDDefaultDTDParser" ) );
    return result;
  }


  public static void main(String[] args)
  {
    junit.textui.TestRunner.run(TestDTDParser.class);
  }
}
