package au.com.explodingsheep.diskDOM.highLevelTests;


import java.util.Calendar;
import org.w3c.dom.*;
import javax.xml.parsers.*;
import au.com.explodingsheep.diskDOM.*;
import au.com.explodingsheep.diskDOM.identifier.*;
import au.com.explodingsheep.diskDOM.documentBuilder.*;


/**
 * Times how long it take the IdentifierListStore to
 * process 1000 lists, then 10000 lists, and sees if
 * there is any slowdown.
 */

public class TimeIdentifierListStore
{
  /**
   * Creates the passed number of IdentifierLists instances
   * and fills them with test data.
   * Returns the number of millisseconds it took to complete the task.
   */
  
  public static int createIdentifierLists( IdentifierListStore ils, int numIdentifierListsToCreate ) throws Exception
  {
    Calendar calendar = Calendar.getInstance();
    long startMilliseconds = calendar.getTimeInMillis();

    IdentifierList ilMain = ils.createIdentifierList();
    
    for ( int loop = 0; loop < numIdentifierListsToCreate; loop++ )
    {
      IdentifierList il = ils.createIdentifierList();
      
      int numIdentifiersToCreate = loop % 10;
      for ( int loop2 = 0; loop2 < numIdentifiersToCreate; loop2++ )
      {
        Identifier i = new IntIdentifier( loop );
        il.add( i );
        ilMain.add( i );
        
        ilMain.get( loop );
      }
    }
    
    calendar = Calendar.getInstance();
    long endMilliseconds = calendar.getTimeInMillis();
    
    int result = ( int ) ( endMilliseconds - startMilliseconds );
    return result;
  }
  
  
  public static void main( String args[] )
  {
    try
    {
      DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
      MyDocumentBuilder db = ( MyDocumentBuilder ) dbf.newDocumentBuilder();
      Document doc = db.newDocument();
      
      MyDOMImplementation di = ( MyDOMImplementation ) db.getDOMImplementation();
      IdentifierListStore ils = di.getIdentifierListStore();
      
      
      int milliseconds1 = createIdentifierLists( ils, 1000 );
      System.out.println( "Time1 = " + milliseconds1 );
      
      int milliseconds2 = createIdentifierLists( ils, 10000 );
      System.out.println( "Time2 = " + milliseconds2 );
      
      float f1 = milliseconds2 / 10;
      float f2 = f1 / ( float ) milliseconds1;
      System.out.println( "Slowdown = " + f2 );
    }
    catch ( Exception e )
    {
      e.printStackTrace();
    }
  }
}
