package au.com.explodingsheep.diskDOM.identifier;


import au.com.explodingsheep.diskDOM.helpers.IntToBytes;
import au.com.explodingsheep.diskDOM.simpleFileSystem.*;


/**
 * A specialized instance of an IdentifierStore that stores
 * IntIdentfiiers to a file. If an Identifier that is not a
 * IntIdentifier is passed to a method here bad things will happen.
 */

public class FileIntIdentifierStore implements IdentifierStore
{
  /* Where everything reads and writes its data to. */
  protected SimpleFileSystem sfs = null;
  
  /* Handle to the file in the sfs above. */
  protected SimpleFileSystemHandle handle = null;
  
  /* Cache used to slightly speed up int to byte conversions. */
  protected byte[] fourBytes = new byte[4];
  

  /**
   * Constructor.
   */
  
  public FileIntIdentifierStore( SimpleFileSystem sfs ) throws IdentifierStoreException
  {
    this.sfs = sfs;
    try
    {
      this.handle = this.sfs.openFile( "FileIntIdentifierStore" );
    }
    catch ( SimpleFileSystemException sfse )
    {
      throw new IdentifierStoreException( sfse );
    }
  }
  
  
  /*----------------------------- IdentifierStore methods ------------------------*/

  
  /**
   * Writes a new Identifier to the store.
   */
  
  public IdentifierIdentifier appendIdentifier( Identifier identifier ) throws IdentifierStoreException
  {
    IdentifierIdentifier result = null;
    IntIdentifier intIdentifier = ( IntIdentifier ) identifier;
    int value = -1;
    if ( intIdentifier != null )
    {
      value = intIdentifier.getValue();
    }
    
    /* Write the value to the end of the file. */
    try
    {
      IntToBytes.intToBytes( value, this.fourBytes );
      int length = this.sfs.size( this.handle );
      this.sfs.seek( this.handle, length );
      this.sfs.write( this.handle, this.fourBytes );
      
      result = new IntIdentifierIdentifier( ( int ) length );
    }
    catch ( SimpleFileSystemException sfse )
    {
      throw new IdentifierStoreException( sfse );
    }
    
    return result;
  }    
      
  
  /**
   * Updates an Identifier in the store.
   */
  
  public void setIdentifier( IdentifierIdentifier ii, Identifier identifier ) throws IdentifierStoreException
  {
    if ( ii != null )
    {
      IntIdentifierIdentifier iii = ( IntIdentifierIdentifier ) ii;
      int offset = iii.getValue();
      
      int value = -1;
      if  ( identifier != null )
      {
        IntIdentifier intIdentifier = ( IntIdentifier ) identifier;
        value = intIdentifier.getValue();
      }
      
      /* Write the value to the end of the file. */
      try
      {
        IntToBytes.intToBytes( value, this.fourBytes );
        this.sfs.seek( this.handle, offset );
        this.sfs.write( this.handle, this.fourBytes );
      }
      catch ( SimpleFileSystemException sfse )
      {
        throw new IdentifierStoreException( sfse );
      }
    }
  }    

  
  
  /**
   * Retrieves an Identifier.
   */
  
  public Identifier getIdentifier( IdentifierIdentifier ii ) throws IdentifierStoreException
  {
    Identifier result = null;
    IntIdentifierIdentifier iii = ( IntIdentifierIdentifier ) ii;
    int offset = iii.getValue();
    
    /* Write the value to the end of the file. */
    try
    {
      this.sfs.seek( this.handle, offset );
      this.sfs.read( this.handle, this.fourBytes );
      int i = IntToBytes.bytesToInt( this.fourBytes );
      if ( i != -1 )
      {
        result = new IntIdentifier( i );
      }
    }
    catch ( SimpleFileSystemException sfse )
    {
      throw new IdentifierStoreException( sfse );
    }
    return result;
  }    
  
  
  /**
   * Closes the store.
   */
  
  public void close() throws IdentifierStoreException
  {
    try
    {
      this.sfs.closeFile( this.handle );
    }
    catch ( SimpleFileSystemException sfse )
    {
      throw new IdentifierStoreException( sfse );
    }
  }
}
