package au.com.explodingsheep.diskDOM.identifier;


import java.io.Serializable;


/**
 * Defines a list of Identifiers. The list should
 * be persistent - a serialized IdentifierList should
 * be able to de-serialize itself and show the Identifiers
 * that were in it prior to Serialization. 
 * IdentifierLists should point to one common pool of 
 * information. If there are two IdentifierLists showing
 * the same list, and one adds or deletes an Identifier
 * the other should reflect those changes.
 * 
 * Because a MyNode is Serializable we are Making Identifiers
 * and IdentifierLists Serializable as well.
 */

public interface IdentifierList extends Serializable
{
  /**
   * Initializes the IdentifierList.
   * Should be called if and when this class is
   * de-serialized.
   */
  
  public void init() throws IdentifierListException;


  /**
   * Returns an Identifier that uniquely identifies this IdentifierList
   * instance.
   */
  
  public Identifier getIdentifier();
  

  /**
   * Returns the item whose index is passed.
   */
  
  public Identifier get( int index ) throws IdentifierListException;
  

  /**
   * Overwrites the value at the passed index with the new value.
   */

  public void set( int index, Identifier identifier ) throws IdentifierListException;

  
  /**
   * Adds an item to the end of the list.
   */
  
  public void add( Identifier identifier ) throws IdentifierListException;
  
  
  /**
   * Removes the passed Identifier from the list.
   */
  
  public void remove( Identifier identifier ) throws IdentifierListException;


  /**
   * Removes the item whose index is passed.
   */
  
  public void remove( int index ) throws IdentifierListException;
  
  
  /**
   * Inserts an item before the passed index item.
   */
  
  public void insert( int index, Identifier identifier ) throws IdentifierListException;


  /**
   * Returns the number of ints in the list.
   */
  
  public int size() throws IdentifierListException;
  
  
  /**
   * Returns the index of the passed int in this list. Returns -1
   * if the int isn't in the list.
   */
  
  public int indexOf( Identifier identifier ) throws IdentifierListException;
}
