package au.com.explodingsheep.diskDOM.identifier;


/**
 * This is where IdentifierLists read and write their Identifiers to.
 * It is one shared common spot for IdentifierLists to read and write 
 * their data to.
 */

public interface IdentifierListStore
{
  /**
   * Creates a new IdentifierList and returns it. The list is
   * initially empty.
   */
  
  public IdentifierList createIdentifierList() throws IdentifierListException;

  
  /**
   * Returns the value of the block whose index is passed.
   * Returns IntListManager if index is out of range.
   */
  
  public Identifier get( IdentifierList il, int index ) throws IdentifierListException;


  /**
   * Sets the the value of the block whose index is passed.
   * Returns IntListManager if index is out of range.
   */
  
  public void set( IdentifierList il, int index, Identifier i ) throws IdentifierListException;
  
  
  /**
   * Deletes the current value in the IntList.
   * The current value becomes the next value in the list.
   */
  
  public void remove( IdentifierList il, int index ) throws IdentifierListException;

  
  /**
   * Deletes the current value in the IntList.
   * The current value becomes the next value in the list.
   */
  
  public void remove( IdentifierList il, Identifier identifier ) throws IdentifierListException;
  
  
  /**
   * Appends a value to the end of the IntList.
   */
  
  public void add( IdentifierList il, Identifier identifier ) throws IdentifierListException;

  
  /**
   * Inserts a value before the current item in the IntList.
   */
  
  public void insert( IdentifierList il, int index, Identifier identifier ) throws IdentifierListException;


  /**
   * Returns the number of items in the IntList.
   */
  
  public int size( IdentifierList il ) throws IdentifierListException;
  
    
  /**
   * Returns the index position of the block with the passed
   * integer, or -1 if it is not in the list.
   */
  
  public int indexOf( IdentifierList il, Identifier identifier ) throws IdentifierListException;
  
  
  /**
   * Closes this object.
   */
  
  public void close() throws IdentifierListException;
}
