package au.com.explodingsheep.diskDOM.identifier;


import au.com.explodingsheep.diskDOM.*;


/**
 * Implements the IdentifierList interface and assumes that IntIdentifiers
 * are what are being loaded and stored. The int value in the IntIdentifiers
 * are stored to a file via another class called 
 * SimpleFileSystemIntIdentifierListManager.
 * This class means external code doesn't need to know about how and where 
 * the Identifiers are stored.
 */

public class IntIdentifierList implements IdentifierList
{
  /* The instance identifier. */
  protected Identifier domImplementationIdentifier;

  /* The IntListManager this class goes to to retrieve */
  /* and store values.                                  */
  protected transient IdentifierListStore ils = null;

  /* Identifies this instance of this class. */
  protected Identifier instanceIdentifier = null;


  /**
   * Constructor.
   */
  
  public IntIdentifierList( Identifier domImplementationIdentifier )
  {
    this.domImplementationIdentifier = domImplementationIdentifier;
  }


  /**
   * Constructor.
   */
  
  public IntIdentifierList( IdentifierListStore ils, Identifier domImplementationIdentifier, Identifier instanceIdentifier )
  {
    this.ils = ils;
    this.domImplementationIdentifier = domImplementationIdentifier;
    this.instanceIdentifier = instanceIdentifier;
  }
  

  /**
   * Constructor.
   */
  
  public IntIdentifierList( Identifier domImplementationIdentifier, Identifier instanceIdentifier )
  {
    this.domImplementationIdentifier = domImplementationIdentifier;
    this.instanceIdentifier = instanceIdentifier;
  }
  
    
  /**
   * Returns the DOMImplementation identifier.
   */
  
  public Identifier getDOMImplementationIdentifier()
  {
    return this.domImplementationIdentifier;
  }


  /*------------------------- IdentifierList methods. ----------------------------*/


  /**
   * Initializes this IntList instance.
   * Should be called if and when this class is
   * de-serialized.
   */
  
  public void init() throws IdentifierListException
  {
    /* Goto the FileIntIdentifierListManagerFactory and get the correct */
    /* instance we need to run.                                          */
    this.ils = MyDOMImplementationManager.getInstance( domImplementationIdentifier ).getIdentifierListStore();
  }


  /**
   * Returns an Identifier that uniquely identifies this IdentifierList
   * instance.
   */
  
  public Identifier getIdentifier()
  {
    return this.instanceIdentifier;
  }


  /**
   * Returns the item whose index is passed.
   * Note that we use the first/next methods to get the
   * item we want as we don't have a quick way to move
   * directly to an item in the IntList.
   */
  
  public Identifier get( int index ) throws IdentifierListException
  {
    Identifier result = this.ils.get( this, index );
    return result;
  }


  /**
   * Sets a value in this list, overwriting what was
   * already there. Appends a value to the end of the list
   * if "index" is greater than the number of items in the list.
   */
  
  public void set( int index, Identifier value ) throws IdentifierListException
  {
    this.ils.set( this, index, value );
  }
  
  
  /**
   * Adds an item to the end of the list.
   */
  
  public void add( Identifier value ) throws IdentifierListException
  {
    this.ils.add( this, value );
  }
  
  
  /**
   * Removes the current item.
   */
  
  public void remove( Identifier value ) throws IdentifierListException
  {
    this.ils.remove( this, ( IntIdentifier ) value );
  }
  
  
  /**
   * Removes the current item.
   */
  
  public void remove( int index ) throws IdentifierListException
  {
    this.ils.remove( this, index );
  }
  
  
  /**
   * Inserts an item before the item whose index is passed.
   */
  
  public void insert( int index, Identifier value ) throws IdentifierListException
  {
    this.ils.insert( this, index, value );
  }
  

  /**
   * Returns the number of ints in the list.
   */
  
  public int size() throws IdentifierListException
  {
    int result = this.ils.size( this );
    return result;
  }
  
  
  /**
   * Returns the index of the passed int in this list. Returns -1
   * if the int isn't in the list.
   */
  
  public int indexOf( Identifier value ) throws IdentifierListException
  {
    int result = this.ils.indexOf( this, value );
    return result;
  }
}
