package au.com.explodingsheep.diskDOM.identifier;


import au.com.explodingsheep.diskDOM.simpleFileSystem.*;
import au.com.explodingsheep.diskDOM.helpers.IntToBytes;


/**
 * An IdentifierStore that stores its identifiers to a
 * SimpleFileSystem. It assumes the identifiers are ints.
 */

public class SimpleFileSystemIdentifierStore implements IdentifierStore
{
  /* This is a null Identifier value. */
  public static final int NULL_IDENTIFIER = -1;
  
  /* File handle to where we store the identifer identifiers. */
  protected SimpleFileSystemHandle handle = null;
  
  /* Where we read and wirte the IdentifierLists to. */
  protected SimpleFileSystem sfs = null;  

  /* Cache used when reading and writing integers to disk. */
  protected byte[] fourBytes = new byte[4];


  /**
   * Constructor.
   */
  
  public SimpleFileSystemIdentifierStore( SimpleFileSystem sfs ) throws SimpleFileSystemException
  {
    if ( sfs != null )
    {
      this.sfs = sfs;
      this.handle = this.sfs.openFile( "SimpleFileSystemIdentifierStore" );
    }
    else
    {
      throw new SimpleFileSystemException( "Passed SimpleFileSystem cannot be null." );
    }
  }


  
  /*------------------- IdentifierIdentifierStore methods ---------------------*/
  
  
  /**
   * Writes a new Identifier to the store.
   */
  
  public IdentifierIdentifier appendIdentifier( Identifier i ) throws IdentifierStoreException
  {
    IdentifierIdentifier result = null;
    
    int value = SimpleFileSystemIdentifierStore.NULL_IDENTIFIER;
    IntIdentifier intIdentifier = ( IntIdentifier ) i;
    if ( intIdentifier != null )
      value = intIdentifier.getValue();
    
    IntToBytes.intToBytes( value, this.fourBytes );
    
    try
    {
      int fileSize = this.sfs.size( this.handle );
      this.sfs.seek( this.handle, fileSize );
      this.sfs.write( this.handle, this.fourBytes );
      
      int intIdentifierIdentifierHandle = fileSize / 4;
      result = new IntIdentifierIdentifier( intIdentifierIdentifierHandle );
    }
    catch ( SimpleFileSystemException sfse )
    {
      throw new IdentifierStoreException( sfse );
    }
    
    return result;
  }
  
  
  /**
   * Updates an Identifier in the store.
   */
  
  public void setIdentifier( IdentifierIdentifier ii, Identifier identifier ) throws IdentifierStoreException
  {
    IdentifierIdentifier result = null;

    try
    {
      IntIdentifierIdentifier iii = ( IntIdentifierIdentifier ) ii;
      int intIdentifierIdentifierHandle = iii.getValue();
  
      int offsetIntoFile = intIdentifierIdentifierHandle * 4;
       this.sfs.seek( this.handle, offsetIntoFile );
  
      int value = SimpleFileSystemIdentifierStore.NULL_IDENTIFIER;
      if ( identifier != null )
      {
        IntIdentifier intIdentifier = ( IntIdentifier ) identifier;
        value = intIdentifier.getValue();
      }
      IntToBytes.intToBytes( value, this.fourBytes );
      
  
      this.sfs.write( this.handle, this.fourBytes );
    }
    catch ( SimpleFileSystemException sfse )
    {
      throw new IdentifierStoreException( sfse );
    }
  }
    
  
  
  /**
   * Retrieves an Identifier.
   */
  
  public Identifier getIdentifier( IdentifierIdentifier ii ) throws IdentifierStoreException
  {
    Identifier result = null;

    try
    {
      IntIdentifierIdentifier iii = ( IntIdentifierIdentifier ) ii;
      int intIdentifierIdentifierHandle = iii.getValue();
  
      int offsetIntoFile = intIdentifierIdentifierHandle * 4;
       this.sfs.seek( this.handle, offsetIntoFile );
       this.sfs.read( this.handle, this.fourBytes );
       int value = IntToBytes.bytesToInt( this.fourBytes );
       if ( value != SimpleFileSystemIdentifierStore.NULL_IDENTIFIER )
         result = new IntIdentifier( value );
    }     
    catch ( SimpleFileSystemException sfse )
    {
      throw new IdentifierStoreException( sfse );
    }
    
      return result;
  }


  /** 
   * Closes this object.
   */
  
  public void close() throws IdentifierStoreException
  {
    try
    {
      this.sfs.closeFile( this.handle );
    }
    catch ( SimpleFileSystemException sfse )
    {
      throw new IdentifierStoreException( sfse );
    }
  }
}
