package au.com.explodingsheep.diskDOM.identifier;


import au.com.explodingsheep.diskDOM.simpleFileSystem.*;
import au.com.explodingsheep.diskDOM.helpers.IntToBytes;


/**
 * Creates unique IntIdentifiers. Stores the last integer used when
 * creating the identifier to the passed SimpleFileSystem.
 */

public class SimpleFileSystemIntIdentifierGenerator implements IdentifierGenerator
{
  static int count = 0;

  /* Cache used when reading and writing integers to disk. */
  protected byte[] fourBytes = new byte[4];

  /* Where everything reads and writes its data to. */
  protected SimpleFileSystem sfs = null;


  /**
   * Constructor.
   */
  
  public SimpleFileSystemIntIdentifierGenerator( SimpleFileSystem sfs ) throws SimpleFileSystemException
  {
    this.sfs = sfs;
    SimpleFileSystemHandle handle = this.sfs.openFile( "SimpleFileSystemIntIdentifierGenerator" );
    try
    {
      if ( this.sfs.size( handle ) == 4 )
      {
        this.sfs.read( handle, this.fourBytes );
        SimpleFileSystemIntIdentifierGenerator.count = IntToBytes.bytesToInt( this.fourBytes );
      }
    }
    finally
    {
      this.sfs.closeFile( handle );
    }
  }


  /*--------------------- IdentifierGenerator methods ----------------------*/
  
    
  /**
   * Returns a new unique Identifier.
   */
  
  public Identifier createIdentifier()
  {
    count++;
    Identifier result = new IntIdentifier( count );
    return result;
  }
  
  
  /**
   * Closes this object.
   */
  
  public void close() throws IdentifierGeneratorException
  {
    try
    {
      SimpleFileSystemHandle handle = this.sfs.openFile( "SimpleFileSystemIntIdentifierGenerator" );
      try
      {
        IntToBytes.intToBytes( SimpleFileSystemIntIdentifierGenerator.count, this.fourBytes );
        this.sfs.write( handle, this.fourBytes );
      }
      finally
      {
        this.sfs.closeFile( handle );
      }
    }
    catch ( SimpleFileSystemException sfse )
    {
      throw new IdentifierGeneratorException( sfse );
    }    
  }
}
