package au.com.explodingsheep.diskDOM.identifier;


import junit.framework.*;
import au.com.explodingsheep.diskDOM.simpleFileSystem.*;


/**
 * @author dcaddy
 *
 * To change this generated comment edit the template variable "typecomment":
 * Window>Preferences>Java>Templates.
 * To enable and disable the creation of type comments go to
 * Window>Preferences>Java>Code Generation.
 */

public class TestIdentifierList extends TestCase
{

  /**
   * Constructor for TestIdentifierList.
   * @param arg0
   */
  public TestIdentifierList(String arg0)
  {
    super(arg0);
  }


  /**
   * Makes sure there are 3 items in the passed IdentifierList
   * and they are the same as those passed.
   */
  
  protected boolean checkList( IdentifierList il, int i1, int i2, int i3, int i4, int i5 ) throws Exception
  {
    boolean result = false;
    if ( il.size() == 5 )
    {
      IntIdentifier ii1 = ( IntIdentifier ) il.get( 0 );
      IntIdentifier ii2 = ( IntIdentifier ) il.get( 1 );
      IntIdentifier ii3 = ( IntIdentifier ) il.get( 2 );
      IntIdentifier ii4 = ( IntIdentifier ) il.get( 3 );
      IntIdentifier ii5 = ( IntIdentifier ) il.get( 4 );
      if ( ( ii1.getValue() == i1 )
        && ( ii2.getValue() == i2 )
        && ( ii3.getValue() == i3 )
        && ( ii4.getValue() == i4 )
        && ( ii5.getValue() == i5 ) )
      {
        result = true;
      }
    }
    return result;
  }


  /**
   * Creates an IdentifierList using the passed values to
   * go into IntIdentifiers.
   */
  
  protected IdentifierList createList( IdentifierListStore ils, int i1, int i2, int i3, int i4, int i5 ) throws Exception
  {
    IdentifierList result = ils.createIdentifierList();
    IntIdentifier ii = new IntIdentifier( i1 );
    result.add( ii );
    ii = new IntIdentifier( i2 );
    result.add( ii );
    ii = new IntIdentifier( i3 );
    result.add( ii );
    ii = new IntIdentifier( i4 );
    result.add( ii );
    ii = new IntIdentifier( i5 );
    result.add( ii );
    return result;
  }


  /**
   * Creates an IdentifierListStore object for use in the various tests.
   */
  
  protected IdentifierListStore createIdentifierListStore() throws Exception
  {
    BlockIO blockIO = new RAMBlockIO();
    SimpleFileSystem sfs = new DefaultSimpleFileSystem( blockIO );
    IdentifierListStore result = new SimpleFileSystemIntIdentifierListStore( sfs, null );
    return result;
  }


  public void testOneSmallList() throws Exception
  {
    boolean result = false;

    IntIdentifier ii = new IntIdentifier( 1 );

    IdentifierListStore ilm = this.createIdentifierListStore();
    
    IdentifierList il1 = this.createList( ilm, 1, 2, 3, 4, 5 );
    
    if ( this.checkList( il1, 1, 2, 3, 4, 5 ) == true )
    {
      Identifier i = il1.get( 0 );
      il1.remove( 0 );
      if ( il1.size() == 4 )
      {
        il1.insert( 2, i );
        if ( this.checkList( il1, 2, 3, 1, 4, 5 ) == true )
        {
          result = true;
        }
      }
    }
    
    ilm.close();
    
    super.assertTrue( result );
  }


  public void testThreeSmallLists() throws Exception
  {
    boolean result = false;

    IntIdentifier ii = new IntIdentifier( 1 );

    IdentifierListStore ilm = this.createIdentifierListStore();
    
    IdentifierList il1 = this.createList( ilm, 1, 2, 3, 4, 5 );
    IdentifierList il2 = this.createList( ilm, 6, 7, 8, 9, 10 );
    IdentifierList il3 = this.createList( ilm, 9, 8, 7, 6, 5 );
    
    if ( ( this.checkList( il1, 1, 2, 3, 4, 5 ) == true )
      && ( this.checkList( il2, 6, 7, 8, 9, 10 ) == true )
      && ( this.checkList( il3, 9, 8, 7, 6, 5 ) == true ) )
    {
      Identifier i = il1.get( 0 );
      il1.remove( 0 );
      if ( il1.size() == 4 )
      {
        il1.insert( 2, i );
        if ( this.checkList( il1, 2, 3, 1, 4, 5 ) == true )
        {
          result = true;
        }
      }
    }
    
    ilm.close();
    
    super.assertTrue( result );
  }


  public void testOneBigList() throws Exception
  {
    boolean result = false;

    IdentifierListStore ilm = this.createIdentifierListStore();
      
    IdentifierList il = ilm.createIdentifierList();
    for ( int loop = 0; loop < 10000; loop++ )
    {
      Identifier i = new IntIdentifier( loop );
      il.add( i );
    }
    
    if ( il.size() == 10000 )
    {
      result = true;
      for ( int loop = 0; loop < 10000; loop++ )
      {
        IntIdentifier ii = ( IntIdentifier ) il.get( loop );
        int value = ii.getValue();
        if ( value != loop )
        {
          System.out.println( loop );
          result = false;
        }
      }
      if ( result == true )
      {
        il.remove( 500 );
        if ( il.size() == 9999 )
        {
          Identifier i = new IntIdentifier( 123 );
          if ( il.indexOf( i ) == 123 )
          {
            il.remove( i );
            if ( il.size() == 9998 )
            {
              if ( il.indexOf( i ) == -1 )
              {
                il.insert( 123, i );
                if ( il.indexOf( i ) == 123 )
                {
                  result = true;
                }
              }
            }
          }
        }
      }
    }
        
    ilm.close();
        
    super.assertTrue( result );
  }


  public void testCreateTwoListButOnlyUseOne() throws Exception
  {
    boolean result = false;

    IdentifierListStore ilm = this.createIdentifierListStore();
    
    IdentifierList il1 = ilm.createIdentifierList();
    IdentifierList il2 = ilm.createIdentifierList();
    
    il2.add( new IntIdentifier( 4 ) );
    il2.add( new IntIdentifier( 5 ) );
    il2.add( new IntIdentifier( 6 ) );

    il1.add( new IntIdentifier( 1 ) );
    il1.add( new IntIdentifier( 2 ) );
    il1.add( new IntIdentifier( 3 ) );

    IntIdentifier i = ( IntIdentifier ) il1.get( 0 );
    if ( i.getValue() == 1 )
    {
      i = ( IntIdentifier ) il1.get( 1 );
      if ( i.getValue() == 2 )
      {
        i = ( IntIdentifier ) il1.get( 2 );
        if ( i.getValue() == 3 )
        {
          i = ( IntIdentifier ) il2.get( 0 );
          if ( i.getValue() == 4 )
          {
            i = ( IntIdentifier ) il2.get( 1 );
            if ( i.getValue() == 5 )
            {
              i = ( IntIdentifier ) il2.get( 2 );
              if ( i.getValue() == 6 )
              {
                result = true;
              }
            }
          }
        }
      }
    }
      
    ilm.close();
        
    super.assertTrue( result );
  }
    
  

  public static Test suite()
  {
    TestSuite result = new TestSuite();
    result.addTest( new TestIdentifierList( "testOneSmallList" ) );
    result.addTest( new TestIdentifierList( "testThreeSmallLists" ) );
    result.addTest( new TestIdentifierList( "testOneBigList" ) );
    result.addTest( new TestIdentifierList( "testCreateTwoListButOnlyUseOne" ) );
    return result;
  }
  

  public static void main(String[] args)
  {
    junit.textui.TestRunner.run(TestIdentifierList.class);
  }
}
