package au.com.explodingsheep.diskDOM.nodeStore;


import au.com.explodingsheep.diskDOM.*;
import au.com.explodingsheep.diskDOM.identifier.*;


/**
 * This interface is used by MyNode to load and save MyNode
 * instances to a persistent spot. It knows nothing about the
 * contents of the Nodes, only that they can be loaded and
 * saved using an Identifier.
 * 
 * The NodeStore is responsible for creating Identifier instances
 * which is assigns to a MyNode when "appendNode" is called.
 */

public interface NodeStore
{
  /**
   * Writes a MyNode, assigning it an Identifier in the process.
   */
  
  public Identifier appendNode( MyNode node ) throws NodeStoreException;
  

  /**
   * Load a node from where ever and returns it.
   * After loading the MyNode it's "init" method is called.
   */
  
  public MyNode loadNode( Identifier identifier ) throws NodeStoreException;
  
  
  /**
   * Writes back the passed MyNode, overwriting the previous
   * MyNode that was stored in the NodeStore somewhere. The Identifier
   * shouldn't change as a result of this update.
   */
  
  public void updateNode( MyNode node ) throws NodeStoreException;
  
  
  /**
   * Deletes the passed node from the NodeStore.
   */
  
  public void deleteNode( Identifier identifier ) throws NodeStoreException;
  
  
  /**
   * Called when we want to close down a NodeStore.
   */
  
  public void close() throws NodeStoreException;
}
