package au.com.explodingsheep.diskDOM.simpleFileSystem;


/**
 * Defines a way of reading and writing to data to some form
 * of persistent storage device like a file or a socket.
 * Block numbers start from zero.
 * It is assumed that the size of each block is fixed, and the
 * same as all other blocks.
 */

public interface BlockIO
{
  /**
   * Returns the size of one block.
   */
  
  public int getBlockSize();
  

  /**
   * Reads data from the passed block. If the data is longer than the
   * block the next linked block will also be read until all the data
   * is read.
   */
  
  public void readBlock( int blockNum, byte[] data ) throws BlockIOException;
  
  
  /**
   * Writes data to the passed block. If the data is larger than the
   * block size then the data will span multiple blocks.
   */
  
  public void writeBlock( int blockNum, byte[] data ) throws BlockIOException;
  
  
  /**
   * Closes this object.
   */
  
  public void close() throws BlockIOException;
  
  
  /**
   * Flushes any cached blocks to disk.
   */
  
  public void flush() throws BlockIOException;
  
  
  /**
   * Returns the number of blocks currently available.
   */
  
  public int getNumBlocks();
  
  
  /**
   * Set the number of blocks available.
   */
  
  public void setNumBlocks( int numBlocks ) throws BlockIOException;
}
