package au.com.explodingsheep.diskDOM.simpleFileSystem;


/**
 * This class is used by SimpleFileSystem instances to keep track of
 * which blocks have been allocated or not.
 */

public interface BlockMapManager
{
  /**
   * Locks a block.
   */

  public void lockBlock( int blockNum ) throws BlockMapManagerException;


  /**
   * Unlocks a block.
   */

  public void unlockBlock( int blockNum ) throws BlockMapManagerException;


  /**
   * Is a block locked?
   */

  public boolean isBlockLocked( int blockNum ) throws BlockMapManagerException;


  /**
   * Finds the next free block in the map.
   */

  public int getFreeBlockNum() throws BlockMapManagerException;


  /**
   * Returns the number of blocks in the blockmap.
   */

  public int getNumBlocks() throws BlockMapManagerException;


  /**
   * Sets the number of available blocks in the map.
   * If the number of blocks is increased then the old
   * block information is retained.
   */

  public void setNumBlocks( int numBlocks ) throws BlockMapManagerException;


  /**
   * Closes this object, saving the blockmap information.
   */

  public void close() throws BlockMapManagerException;
}
