package au.com.explodingsheep.diskDOM.simpleFileSystem;


/**
 * This defines a simple file system. It allows you to
 * create and open a file and read and write to it.
 * Files don't have directories.
 */

public interface SimpleFileSystem
{
  /* Where to seek to. */
  public static int SEEK_START    = 1;
  public static int SEEK_END      = 2;
  public static int SEEK_RELATIVE = 3;

  
  /**
   * Deletes a file. If the file doesn't exist then do nothing.
   */

  public void deleteFile( String fileName ) throws SimpleFileSystemException;


  /**
   * Opens a file. If it doesn't exist then create it.
   */

  public SimpleFileSystemHandle openFile( String fileName ) throws SimpleFileSystemException;


  /**
   * Closes a file.
   */

  public void closeFile( SimpleFileSystemHandle handle ) throws SimpleFileSystemException;
  
  
  /**
   * Shuts down the SimpleFileSystem instance.
   */
  
  public void close() throws SimpleFileSystemException;
  
  
  /**
   * Returns the size of the file.
   */
  
  public int size( SimpleFileSystemHandle handle ) throws SimpleFileSystemException;
  
  
  /**
   * Moves the current file position to that passed.
   * 
   * @param startPosition one of SEEK_START, SEEK_RELATIVE or SEEK_END.
   * @param distance the distance to seek relative to startPosition.
   */
  
  public void seek( SimpleFileSystemHandle handle, int startPosition, int distance ) throws SimpleFileSystemException;


  /**
   * Moves the current file position to that passed.
   * 
   * @param distance the distance to seek relative to startPosition.
   */
  
  public void seek( SimpleFileSystemHandle handle, int distance ) throws SimpleFileSystemException;


  /**
   * Reads data from the file.
   */
  
  public int read( SimpleFileSystemHandle handle, byte[] data ) throws SimpleFileSystemException;


  /**
   * Write data to the file.
   */
  
  public void write( SimpleFileSystemHandle handle, byte[] data ) throws SimpleFileSystemException;
  
  
  /**
   * Does a file exist?
   */
  
  public boolean doesFileExist( String fileName ) throws SimpleFileSystemException;
}
