package au.com.explodingsheep.diskDOM.simpleFileSystem;


/**
 * Implements BlockMapManager, reading and writing its
 * blockmap information to a file in a SimpleFileSystem.
 */

public class SimpleFileSystemBlockMapManager extends DefaultBlockMapManager
{
  /* The name of the file we load and save the blockmap to. */
  protected String fileName = null;

  /* Where we read and write the SimpleFileSystem to. */
  protected SimpleFileSystem sfs = null;


  /**
   * Constructor.
   */

  public SimpleFileSystemBlockMapManager( SimpleFileSystem sfs, String fileName ) throws SimpleFileSystemException, BlockMapManagerException
  {
    super();
    
    this.fileName = fileName;
    this.sfs = sfs;
    
    /* Try to read the blockmap from the SimpleFileSystem. */
    SimpleFileSystemHandle handle = sfs.openFile( fileName );
    try
    {
      int fileLength = sfs.size( handle );
      super.blockMap = new byte[fileLength];
      sfs.read( handle, super.blockMap );
    }
    finally
    {
      sfs.closeFile( handle );
    }
  }


  /*----------------------- Overridden BlockMapManager methods --------------------*/


  /**
   * Closes this object, saving the blockmap information.
   * Note that we are writing the blockmap as a file to the SimpleFileSystem.
   * This in turn allocates blocks in this blockmap. To overcome this
   * we first write a block of "this.blockMap.length" zeros, then we
   * seek back to the start of the file and write the blockmap data.
   * We first write a block of zeros
   */

  public void close() throws BlockMapManagerException
  {
    super.close();
    
    try
    {
      this.sfs.deleteFile( this.fileName );
      byte[] zeroBlock = new byte[this.blockMap.length];
      SimpleFileSystemHandle handle = this.sfs.openFile( this.fileName );
      try
      {
        this.sfs.write( handle, zeroBlock );
        
        /* Now overwrite the zeros with the real blockmap data. */
        this.sfs.seek( handle, SimpleFileSystem.SEEK_START, 0 );
        this.sfs.write( handle, super.blockMap );
      }
      finally
      {
        this.sfs.closeFile( handle );
      }
    }
    catch ( SimpleFileSystemException sfse )
    {
      throw new BlockMapManagerException( sfse );
    }
  }
}
