package au.com.explodingsheep.diskDOM.simpleFileSystem;


import java.util.Arrays;
import java.util.Random;
import java.io.*;
import junit.framework.TestCase;


/**
 * @author Daniel Caddy
 *
 * To change this generated comment edit the template variable "typecomment":
 * Window>Preferences>Java>Templates.
 * To enable and disable the creation of type comments go to
 * Window>Preferences>Java>Code Generation.
 */

public class TestCachedBlockIO extends TestCase
{

  /**
   * Constructor for TestCachedBlockIO.
   * @param arg0
   */
  public TestCachedBlockIO(String arg0)
  {
    super(arg0);
  }



  /**
   * Tests that after assorted blocks have been read and written
   * to the cached block they are also eventually written to the
   * real file block io when everything is closed.
   */
  
  public void testThatTheBlocksEventuallyWriteToTheRealBlockIO() throws Exception
  {
    boolean result = false;
    
    BlockIO fileBlockIO = new FileBlockIO( "/tmp/TestCachedBlockIO.dat" );
    fileBlockIO.setNumBlocks( 1000 );
    BlockIO cachedBlockIO = new CachedBlockIO( fileBlockIO, 64 );
    
    byte[] block = new byte[512];
    for ( int loop = 0; loop < 1000; loop++ )
    {
      Arrays.fill( block, ( byte ) loop );
      cachedBlockIO.writeBlock( loop, block );
    }

    /* Make sure the blocks we have written are still there. */
    result = true;
    byte[] expectedBlock = new byte[512];
    Random rg = new Random();
    for ( int loop = 0; loop < 1000; loop++ )
    {
      int blockNum = rg.nextInt( 1000 );
      Arrays.fill( expectedBlock, ( byte ) blockNum );
      cachedBlockIO.readBlock( blockNum, block );
      if ( Arrays.equals( block, expectedBlock ) == false )
      {
        result = false;
      }
    }

    for ( int loop = 0; loop < 1000; loop++ )
    {
      Arrays.fill( expectedBlock, ( byte ) loop );
      cachedBlockIO.readBlock( loop, block );
      if ( Arrays.equals( block, expectedBlock ) == false )
      {
        result = false;
      }
    }

    cachedBlockIO.close();
    
    if ( result == true )
    {
      /* Now make sure the real block file has the contents we wrote. */
      result = true;
      InputStream is = new FileInputStream( "/tmp/TestCachedBlockIO.dat" );
      for ( int loop = 0; loop < 256; loop++ )
      {
        Arrays.fill( expectedBlock, ( byte ) loop );
        is.read( block );
        if ( Arrays.equals( block, expectedBlock ) == false )
        {
          result = false;
        }
      }
    }

    super.assertTrue( result );
  }
    

  public static void main(String[] args)
  {
    junit.textui.TestRunner.run(TestCachedBlockIO.class);
  }

}
