package au.com.explodingsheep.diskDOM.simpleFileSystem;


import junit.framework.TestCase;


/**
 * Tests instances of the SimpleFileSystem interface.
 */

public class TestSimpleFileSystem extends TestCase
{
  protected BlockIO blockIO = null;
  protected SimpleFileSystem sfs = null;

  /**
   * Constructor for TestSimpleFileSystem.
   * @param arg0
   */
  public TestSimpleFileSystem(String arg0)
  {
    super(arg0);
  }
  
  
  public void setUp() throws Exception
  {
//    this.blockIO = new FileBlockIO( "/tmp/testDefaultSimpleFileSystem.dat" );
    this.blockIO = new RAMBlockIO();
    this.sfs = new DefaultSimpleFileSystem( blockIO );
  }


  public void tearDown() throws Exception
  {
    this.sfs.close();
    this.blockIO.close();
  }


  public void testWriteAndReadOneFile() throws Exception
  {
    boolean result = false;
  
    SimpleFileSystemHandle handle = sfs.openFile( "test" );
    try
    {
      byte[] data = new byte[15000];
      for ( int loop = 0; loop < 15000; loop++ )
      {
        data[loop] = ( byte ) ( loop & 255 );
      }
      
      sfs.write( handle, data );
      
      if ( sfs.size( handle ) == 15000 )
      {
        sfs.seek( handle, SimpleFileSystem.SEEK_START, 0 );
        
        byte[] data2 = new byte[15000];
        sfs.read( handle, data2 );
        
        result = true;
        for ( int loop = 0; loop < data2.length; loop++ )
        {
          if ( data[loop] != data2[loop] )
          {
            result = false;
          }
        }
        
        if ( result == true )
        {
          /* Try seeking to a few different points and see if the data */
          /* at that point is correct.                                 */
          result = false;
          sfs.seek( handle, SimpleFileSystem.SEEK_START, 3 );
          byte[] b = new byte[1];
          sfs.read( handle, b );
          if ( b[0] == 3 )
          {
            result = true;
          }
        }
      }
    }
    finally
    {
      sfs.closeFile( handle );
    }

    super.assertTrue( result );
  }
  
  
  protected boolean doesArrayContain( byte[] array, byte ch )
  {
    boolean result = true;
    
    for ( int loop = 0; loop < array.length; loop++ )
    {
      if ( array[loop] != ch )
        result = false;
    }
    
    return result;
  }


  public void testWriteAndReadMultipleFiles() throws Exception
  {
    boolean result = false;
  
    SimpleFileSystemHandle handle1 = sfs.openFile( "test1" );
    SimpleFileSystemHandle handle2 = sfs.openFile( "test2" );
    SimpleFileSystemHandle handle3 = sfs.openFile( "test3" );
    try
    {
      byte[] data1 = new byte[1000];
      byte[] data2 = new byte[1000];
      byte[] data3 = new byte[1000];
      for ( int loop = 0; loop < 1000; loop++ )
      {
        data1[loop] = 'a';
        data2[loop] = 'b';
        data3[loop] = 'c';
      }
      sfs.write( handle1, data1 );
      sfs.write( handle2, data2 );
      sfs.write( handle3, data3 );
      
      for ( int loop = 0; loop < 1000; loop++ )
      {
        data1[loop] = 'b';
        data2[loop] = 'c';
        data3[loop] = 'a';
      }
      sfs.write( handle1, data1 );
      sfs.write( handle2, data2 );
      sfs.write( handle3, data3 );

      for ( int loop = 0; loop < 1000; loop++ )
      {
        data1[loop] = 'c';
        data2[loop] = 'a';
        data3[loop] = 'b';
      }
      sfs.write( handle1, data1 );
      sfs.write( handle2, data2 );
      sfs.write( handle3, data3 );

      if ( ( sfs.size( handle1 ) == 3000 )
        && ( sfs.size( handle2 ) == 3000 )
        && ( sfs.size( handle3 ) == 3000 ) )
      {
        byte[] data1a = new byte[1000];
        byte[] data2a = new byte[1000];
        byte[] data3a = new byte[1000];

        sfs.seek( handle1, SimpleFileSystem.SEEK_START, 0 );
        sfs.seek( handle2, SimpleFileSystem.SEEK_START, 0 );
        sfs.seek( handle3, SimpleFileSystem.SEEK_START, 0 );

        sfs.read( handle1, data1a );
        sfs.read( handle2, data2a );
        sfs.read( handle3, data3a );
        
        if ( ( this.doesArrayContain( data1a, ( byte ) 'a' ) == true )
          && ( this.doesArrayContain( data2a, ( byte ) 'b' ) == true )
          && ( this.doesArrayContain( data3a, ( byte ) 'c' ) == true ) )
        {
          sfs.read( handle1, data1a );
          sfs.read( handle2, data2a );
          sfs.read( handle3, data3a );
          
          if ( ( this.doesArrayContain( data1a, ( byte ) 'b' ) == true )
            && ( this.doesArrayContain( data2a, ( byte ) 'c' ) == true )
            && ( this.doesArrayContain( data3a, ( byte ) 'a' ) == true ) )
          {
            sfs.read( handle1, data1a );
            sfs.read( handle2, data2a );
            sfs.read( handle3, data3a );
            
            if ( ( this.doesArrayContain( data1a, ( byte ) 'c' ) == true )
              && ( this.doesArrayContain( data2a, ( byte ) 'a' ) == true )
              && ( this.doesArrayContain( data3a, ( byte ) 'b' ) == true ) )
            {
              result = true;
            }
          }
        }
      }
    }
    finally
    {
      sfs.closeFile( handle1 );
      sfs.closeFile( handle2 );
      sfs.closeFile( handle3 );
    }

    super.assertTrue( result );
  }
  

  public void testReadAndWriteReallyBigFile() throws Exception
  {
    boolean result = false;
    
    SimpleFileSystemHandle handle = sfs.openFile( "testReadAndWriteReallyBigFile" );
    try
    {
      byte data[] = new byte[1024];
      
      for ( int loop = 0; loop < 5000; loop++ )
      {
        for ( int loop2 = 0; loop2 < 1024; loop2++ )
        {
          data[loop2] = ( byte ) ( loop & 255 );
        }
        
        sfs.write( handle, data );
      }
      
      if ( sfs.size( handle ) == 5120000 )
      {
        sfs.seek( handle, SimpleFileSystem.SEEK_START, 0 );
        
        result = true;
        for ( int loop = 0; loop < 5000; loop++ )
        {
          sfs.read( handle, data );
          
          if ( this.doesArrayContain( data, ( byte ) ( loop & 255 ) ) == false )
          {
            result = false;
          }
        }
      }
    }
    finally
    {
      sfs.closeFile( handle );
    }
    
    super.assertTrue( result );
  }
    

  public static void main(String[] args)
  {
    junit.textui.TestRunner.run(TestSimpleFileSystem.class);
  }
}
