package au.com.explodingsheep.diskDOM.stringStore;


import java.util.*;
import au.com.explodingsheep.diskDOM.identifier.*;


/**
 * This is a quick and nasty implementation of the StringStore interface
 * so I can quickly test to see if this whole concept of a StringStore works.
 * 
 * Note that we are using to Lists to hold the keys and values as we need
 * to be able to compare to Identifiers using their "equals" method instead
 * of "==", as we may have multiple Identifiers out there that all refer to 
 * the same string.
 */

public class RAMStringStore implements StringStore
{
  /* Where we store the Identifiers to. */
  protected List identifiers = new ArrayList();

  /* Where we store the strings to. */
  protected List strings = new ArrayList();
  
  /* The number of strings in the store.      */
  /* Used to create new IntIdentifiers. */
  int count = 0;
  
  
  /**
   * Constructor.
   */
  
  public RAMStringStore()
  {
  }
    

  /**
   * Returns the position in the list of strings of the
   * passed Identifier, or -1 if it is not in the list.
   */
  
  protected int indexOf( Identifier identifier )
  {
    int result = -1;
    int loop = 0;
    int numIdentifiersInList = this.identifiers.size();
    while ( loop < numIdentifiersInList )
    {
      Identifier i = ( Identifier ) this.identifiers.get( loop );
      if ( i.equals( identifier ) )
      {
        result = loop;
        loop = numIdentifiersInList;
      }
      else
      {
        loop++;
      }
    }
    return result;
  }


  /*---------------------------- StringStore methods --------------------------*/
  
  
  /**
   * Saves a string to some persistent spot, and
   * returns a key that uniquely identifies the string.
   */
  
  public Identifier appendString( String s ) throws StringStoreException
  {
    count++;
    IntIdentifier identifier = new IntIdentifier( count );
    this.identifiers.add( identifier );
    this.strings.add( s );
    return identifier;
  }

  
  /**
   * Saves a string to some persistent spot, and
   * returns a key that uniquely identifies the string.
   */
  
  public void setString( Identifier identifier, String s ) throws StringStoreException
  {
    int index = this.indexOf( identifier );
    if ( index > -1 )
    {
      this.strings.set( index, s );
    }
  }
    

  /**
   * Retrieves a string from the persistent spot.
   */
  
  public String getString( Identifier identifier ) throws StringStoreException
  {
     String result = null;
     int index = this.indexOf( identifier );
     if ( index > -1 )
     {
      result = ( String ) this.strings.get( index );
     }
    return result;
  }
  
    
  /**
   * Closes this object.
   */
  
  public void close() throws StringStoreException
  {
  }
}
